# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import logging as std_logging
import re
from typing import (
    Callable,
    Dict,
    Mapping,
    MutableMapping,
    MutableSequence,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
)

from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry_async as retries
from google.api_core.client_options import ClientOptions
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore
import google.protobuf

from google.cloud.monitoring_v3 import gapic_version as package_version

try:
    OptionalRetry = Union[retries.AsyncRetry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.AsyncRetry, object, None]  # type: ignore

from google.longrunning import operations_pb2  # type: ignore
from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import wrappers_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore

from google.cloud.monitoring_v3.services.alert_policy_service import pagers
from google.cloud.monitoring_v3.types import alert, alert_service, mutation_record

from .client import AlertPolicyServiceClient
from .transports.base import DEFAULT_CLIENT_INFO, AlertPolicyServiceTransport
from .transports.grpc_asyncio import AlertPolicyServiceGrpcAsyncIOTransport

try:
    from google.api_core import client_logging  # type: ignore

    CLIENT_LOGGING_SUPPORTED = True  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    CLIENT_LOGGING_SUPPORTED = False

_LOGGER = std_logging.getLogger(__name__)


class AlertPolicyServiceAsyncClient:
    """The AlertPolicyService API is used to manage (list, create, delete,
    edit) alert policies in Cloud Monitoring. An alerting policy is a
    description of the conditions under which some aspect of your system
    is considered to be "unhealthy" and the ways to notify people or
    services about this state. In addition to using this API, alert
    policies can also be managed through `Cloud
    Monitoring <https://cloud.google.com/monitoring/docs/>`__, which can
    be reached by clicking the "Monitoring" tab in `Cloud
    console <https://console.cloud.google.com/>`__.
    """

    _client: AlertPolicyServiceClient

    # Copy defaults from the synchronous client for use here.
    # Note: DEFAULT_ENDPOINT is deprecated. Use _DEFAULT_ENDPOINT_TEMPLATE instead.
    DEFAULT_ENDPOINT = AlertPolicyServiceClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = AlertPolicyServiceClient.DEFAULT_MTLS_ENDPOINT
    _DEFAULT_ENDPOINT_TEMPLATE = AlertPolicyServiceClient._DEFAULT_ENDPOINT_TEMPLATE
    _DEFAULT_UNIVERSE = AlertPolicyServiceClient._DEFAULT_UNIVERSE

    alert_policy_path = staticmethod(AlertPolicyServiceClient.alert_policy_path)
    parse_alert_policy_path = staticmethod(
        AlertPolicyServiceClient.parse_alert_policy_path
    )
    alert_policy_condition_path = staticmethod(
        AlertPolicyServiceClient.alert_policy_condition_path
    )
    parse_alert_policy_condition_path = staticmethod(
        AlertPolicyServiceClient.parse_alert_policy_condition_path
    )
    common_billing_account_path = staticmethod(
        AlertPolicyServiceClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        AlertPolicyServiceClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(AlertPolicyServiceClient.common_folder_path)
    parse_common_folder_path = staticmethod(
        AlertPolicyServiceClient.parse_common_folder_path
    )
    common_organization_path = staticmethod(
        AlertPolicyServiceClient.common_organization_path
    )
    parse_common_organization_path = staticmethod(
        AlertPolicyServiceClient.parse_common_organization_path
    )
    common_project_path = staticmethod(AlertPolicyServiceClient.common_project_path)
    parse_common_project_path = staticmethod(
        AlertPolicyServiceClient.parse_common_project_path
    )
    common_location_path = staticmethod(AlertPolicyServiceClient.common_location_path)
    parse_common_location_path = staticmethod(
        AlertPolicyServiceClient.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            AlertPolicyServiceAsyncClient: The constructed client.
        """
        return AlertPolicyServiceClient.from_service_account_info.__func__(AlertPolicyServiceAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            AlertPolicyServiceAsyncClient: The constructed client.
        """
        return AlertPolicyServiceClient.from_service_account_file.__func__(AlertPolicyServiceAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return AlertPolicyServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> AlertPolicyServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            AlertPolicyServiceTransport: The transport used by the client instance.
        """
        return self._client.transport

    @property
    def api_endpoint(self):
        """Return the API endpoint used by the client instance.

        Returns:
            str: The API endpoint used by the client instance.
        """
        return self._client._api_endpoint

    @property
    def universe_domain(self) -> str:
        """Return the universe domain used by the client instance.

        Returns:
            str: The universe domain used
                by the client instance.
        """
        return self._client._universe_domain

    get_transport_class = AlertPolicyServiceClient.get_transport_class

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Optional[
            Union[
                str,
                AlertPolicyServiceTransport,
                Callable[..., AlertPolicyServiceTransport],
            ]
        ] = "grpc_asyncio",
        client_options: Optional[ClientOptions] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the alert policy service async client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Optional[Union[str,AlertPolicyServiceTransport,Callable[..., AlertPolicyServiceTransport]]]):
                The transport to use, or a Callable that constructs and returns a new transport to use.
                If a Callable is given, it will be called with the same set of initialization
                arguments as used in the AlertPolicyServiceTransport constructor.
                If set to None, a transport is chosen automatically.
            client_options (Optional[Union[google.api_core.client_options.ClientOptions, dict]]):
                Custom options for the client.

                1. The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client when ``transport`` is
                not explicitly provided. Only if this property is not set and
                ``transport`` was not explicitly provided, the endpoint is
                determined by the GOOGLE_API_USE_MTLS_ENDPOINT environment
                variable, which have one of the following values:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto-switch to the
                default mTLS endpoint if client certificate is present; this is
                the default value).

                2. If the GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide a client certificate for mTLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

                3. The ``universe_domain`` property can be used to override the
                default "googleapis.com" universe. Note that ``api_endpoint``
                property still takes precedence; and ``universe_domain`` is
                currently not supported for mTLS.

            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = AlertPolicyServiceClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

        if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
            std_logging.DEBUG
        ):  # pragma: NO COVER
            _LOGGER.debug(
                "Created client `google.monitoring_v3.AlertPolicyServiceAsyncClient`.",
                extra={
                    "serviceName": "google.monitoring.v3.AlertPolicyService",
                    "universeDomain": getattr(
                        self._client._transport._credentials, "universe_domain", ""
                    ),
                    "credentialsType": f"{type(self._client._transport._credentials).__module__}.{type(self._client._transport._credentials).__qualname__}",
                    "credentialsInfo": getattr(
                        self.transport._credentials, "get_cred_info", lambda: None
                    )(),
                }
                if hasattr(self._client._transport, "_credentials")
                else {
                    "serviceName": "google.monitoring.v3.AlertPolicyService",
                    "credentialsType": None,
                },
            )

    async def list_alert_policies(
        self,
        request: Optional[Union[alert_service.ListAlertPoliciesRequest, dict]] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> pagers.ListAlertPoliciesAsyncPager:
        r"""Lists the existing alerting policies for the
        workspace.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import monitoring_v3

            async def sample_list_alert_policies():
                # Create a client
                client = monitoring_v3.AlertPolicyServiceAsyncClient()

                # Initialize request argument(s)
                request = monitoring_v3.ListAlertPoliciesRequest(
                    name="name_value",
                )

                # Make the request
                page_result = client.list_alert_policies(request=request)

                # Handle the response
                async for response in page_result:
                    print(response)

        Args:
            request (Optional[Union[google.cloud.monitoring_v3.types.ListAlertPoliciesRequest, dict]]):
                The request object. The protocol for the ``ListAlertPolicies`` request.
            name (:class:`str`):
                Required. The
                `project <https://cloud.google.com/monitoring/api/v3#project_name>`__
                whose alert policies are to be listed. The format is:

                ::

                    projects/[PROJECT_ID_OR_NUMBER]

                Note that this field names the parent container in which
                the alerting policies to be listed are stored. To
                retrieve a single alerting policy by name, use the
                [GetAlertPolicy][google.monitoring.v3.AlertPolicyService.GetAlertPolicy]
                operation, instead.

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.monitoring_v3.services.alert_policy_service.pagers.ListAlertPoliciesAsyncPager:
                The protocol for the ListAlertPolicies response.

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [name]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, alert_service.ListAlertPoliciesRequest):
            request = alert_service.ListAlertPoliciesRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.list_alert_policies
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListAlertPoliciesAsyncPager(
            method=rpc,
            request=request,
            response=response,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_alert_policy(
        self,
        request: Optional[Union[alert_service.GetAlertPolicyRequest, dict]] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> alert.AlertPolicy:
        r"""Gets a single alerting policy.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import monitoring_v3

            async def sample_get_alert_policy():
                # Create a client
                client = monitoring_v3.AlertPolicyServiceAsyncClient()

                # Initialize request argument(s)
                request = monitoring_v3.GetAlertPolicyRequest(
                    name="name_value",
                )

                # Make the request
                response = await client.get_alert_policy(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.monitoring_v3.types.GetAlertPolicyRequest, dict]]):
                The request object. The protocol for the ``GetAlertPolicy`` request.
            name (:class:`str`):
                Required. The alerting policy to retrieve. The format
                is:

                ::

                    projects/[PROJECT_ID_OR_NUMBER]/alertPolicies/[ALERT_POLICY_ID]

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.monitoring_v3.types.AlertPolicy:
                A description of the conditions under which some aspect of your system is
                   considered to be "unhealthy" and the ways to notify
                   people or services about this state. For an overview
                   of alerting policies, see [Introduction to
                   Alerting](https://cloud.google.com/monitoring/alerts/).

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [name]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, alert_service.GetAlertPolicyRequest):
            request = alert_service.GetAlertPolicyRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.get_alert_policy
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def create_alert_policy(
        self,
        request: Optional[Union[alert_service.CreateAlertPolicyRequest, dict]] = None,
        *,
        name: Optional[str] = None,
        alert_policy: Optional[alert.AlertPolicy] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> alert.AlertPolicy:
        r"""Creates a new alerting policy.

        Design your application to single-thread API calls that
        modify the state of alerting policies in a single
        project. This includes calls to CreateAlertPolicy,
        DeleteAlertPolicy and UpdateAlertPolicy.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import monitoring_v3

            async def sample_create_alert_policy():
                # Create a client
                client = monitoring_v3.AlertPolicyServiceAsyncClient()

                # Initialize request argument(s)
                request = monitoring_v3.CreateAlertPolicyRequest(
                    name="name_value",
                )

                # Make the request
                response = await client.create_alert_policy(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.monitoring_v3.types.CreateAlertPolicyRequest, dict]]):
                The request object. The protocol for the ``CreateAlertPolicy`` request.
            name (:class:`str`):
                Required. The
                `project <https://cloud.google.com/monitoring/api/v3#project_name>`__
                in which to create the alerting policy. The format is:

                ::

                    projects/[PROJECT_ID_OR_NUMBER]

                Note that this field names the parent container in which
                the alerting policy will be written, not the name of the
                created policy. \|name\| must be a host project of a
                Metrics Scope, otherwise INVALID_ARGUMENT error will
                return. The alerting policy that is returned will have a
                name that contains a normalized representation of this
                name as a prefix but adds a suffix of the form
                ``/alertPolicies/[ALERT_POLICY_ID]``, identifying the
                policy in the container.

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            alert_policy (:class:`google.cloud.monitoring_v3.types.AlertPolicy`):
                Required. The requested alerting policy. You should omit
                the ``name`` field in this policy. The name will be
                returned in the new policy, including a new
                ``[ALERT_POLICY_ID]`` value.

                This corresponds to the ``alert_policy`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.monitoring_v3.types.AlertPolicy:
                A description of the conditions under which some aspect of your system is
                   considered to be "unhealthy" and the ways to notify
                   people or services about this state. For an overview
                   of alerting policies, see [Introduction to
                   Alerting](https://cloud.google.com/monitoring/alerts/).

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [name, alert_policy]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, alert_service.CreateAlertPolicyRequest):
            request = alert_service.CreateAlertPolicyRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name
        if alert_policy is not None:
            request.alert_policy = alert_policy

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.create_alert_policy
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def delete_alert_policy(
        self,
        request: Optional[Union[alert_service.DeleteAlertPolicyRequest, dict]] = None,
        *,
        name: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> None:
        r"""Deletes an alerting policy.

        Design your application to single-thread API calls that
        modify the state of alerting policies in a single
        project. This includes calls to CreateAlertPolicy,
        DeleteAlertPolicy and UpdateAlertPolicy.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import monitoring_v3

            async def sample_delete_alert_policy():
                # Create a client
                client = monitoring_v3.AlertPolicyServiceAsyncClient()

                # Initialize request argument(s)
                request = monitoring_v3.DeleteAlertPolicyRequest(
                    name="name_value",
                )

                # Make the request
                await client.delete_alert_policy(request=request)

        Args:
            request (Optional[Union[google.cloud.monitoring_v3.types.DeleteAlertPolicyRequest, dict]]):
                The request object. The protocol for the ``DeleteAlertPolicy`` request.
            name (:class:`str`):
                Required. The alerting policy to delete. The format is:

                ::

                    projects/[PROJECT_ID_OR_NUMBER]/alertPolicies/[ALERT_POLICY_ID]

                For more information, see
                [AlertPolicy][google.monitoring.v3.AlertPolicy].

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.
        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [name]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, alert_service.DeleteAlertPolicyRequest):
            request = alert_service.DeleteAlertPolicyRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.delete_alert_policy
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

    async def update_alert_policy(
        self,
        request: Optional[Union[alert_service.UpdateAlertPolicyRequest, dict]] = None,
        *,
        update_mask: Optional[field_mask_pb2.FieldMask] = None,
        alert_policy: Optional[alert.AlertPolicy] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> alert.AlertPolicy:
        r"""Updates an alerting policy. You can either replace the entire
        policy with a new one or replace only certain fields in the
        current alerting policy by specifying the fields to be updated
        via ``updateMask``. Returns the updated alerting policy.

        Design your application to single-thread API calls that modify
        the state of alerting policies in a single project. This
        includes calls to CreateAlertPolicy, DeleteAlertPolicy and
        UpdateAlertPolicy.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import monitoring_v3

            async def sample_update_alert_policy():
                # Create a client
                client = monitoring_v3.AlertPolicyServiceAsyncClient()

                # Initialize request argument(s)
                request = monitoring_v3.UpdateAlertPolicyRequest(
                )

                # Make the request
                response = await client.update_alert_policy(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.cloud.monitoring_v3.types.UpdateAlertPolicyRequest, dict]]):
                The request object. The protocol for the ``UpdateAlertPolicy`` request.
            update_mask (:class:`google.protobuf.field_mask_pb2.FieldMask`):
                Optional. A list of alerting policy field names. If this
                field is not empty, each listed field in the existing
                alerting policy is set to the value of the corresponding
                field in the supplied policy (``alert_policy``), or to
                the field's default value if the field is not in the
                supplied alerting policy. Fields not listed retain their
                previous value.

                Examples of valid field masks include ``display_name``,
                ``documentation``, ``documentation.content``,
                ``documentation.mime_type``, ``user_labels``,
                ``user_label.nameofkey``, ``enabled``, ``conditions``,
                ``combiner``, etc.

                If this field is empty, then the supplied alerting
                policy replaces the existing policy. It is the same as
                deleting the existing policy and adding the supplied
                policy, except for the following:

                - The new policy will have the same
                  ``[ALERT_POLICY_ID]`` as the former policy. This gives
                  you continuity with the former policy in your
                  notifications and incidents.
                - Conditions in the new policy will keep their former
                  ``[CONDITION_ID]`` if the supplied condition includes
                  the ``name`` field with that ``[CONDITION_ID]``. If
                  the supplied condition omits the ``name`` field, then
                  a new ``[CONDITION_ID]`` is created.

                This corresponds to the ``update_mask`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            alert_policy (:class:`google.cloud.monitoring_v3.types.AlertPolicy`):
                Required. The updated alerting policy or the updated
                values for the fields listed in ``update_mask``. If
                ``update_mask`` is not empty, any fields in this policy
                that are not in ``update_mask`` are ignored.

                This corresponds to the ``alert_policy`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.monitoring_v3.types.AlertPolicy:
                A description of the conditions under which some aspect of your system is
                   considered to be "unhealthy" and the ways to notify
                   people or services about this state. For an overview
                   of alerting policies, see [Introduction to
                   Alerting](https://cloud.google.com/monitoring/alerts/).

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [update_mask, alert_policy]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, alert_service.UpdateAlertPolicyRequest):
            request = alert_service.UpdateAlertPolicyRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if update_mask is not None:
            request.update_mask = update_mask
        if alert_policy is not None:
            request.alert_policy = alert_policy

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.update_alert_policy
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("alert_policy.name", request.alert_policy.name),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def __aenter__(self) -> "AlertPolicyServiceAsyncClient":
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)

if hasattr(DEFAULT_CLIENT_INFO, "protobuf_runtime_version"):  # pragma: NO COVER
    DEFAULT_CLIENT_INFO.protobuf_runtime_version = google.protobuf.__version__


__all__ = ("AlertPolicyServiceAsyncClient",)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.api import metric_pb2  # type: ignore
from google.api import monitored_resource_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.monitoring_v3.types import common
from google.cloud.monitoring_v3.types import metric as gm_metric

__protobuf__ = proto.module(
    package="google.monitoring.v3",
    manifest={
        "ListMonitoredResourceDescriptorsRequest",
        "ListMonitoredResourceDescriptorsResponse",
        "GetMonitoredResourceDescriptorRequest",
        "ListMetricDescriptorsRequest",
        "ListMetricDescriptorsResponse",
        "GetMetricDescriptorRequest",
        "CreateMetricDescriptorRequest",
        "DeleteMetricDescriptorRequest",
        "ListTimeSeriesRequest",
        "ListTimeSeriesResponse",
        "CreateTimeSeriesRequest",
        "CreateTimeSeriesError",
        "CreateTimeSeriesSummary",
        "QueryTimeSeriesRequest",
        "QueryTimeSeriesResponse",
        "QueryErrorList",
    },
)


class ListMonitoredResourceDescriptorsRequest(proto.Message):
    r"""The ``ListMonitoredResourceDescriptors`` request.

    Attributes:
        name (str):
            Required. The
            `project <https://cloud.google.com/monitoring/api/v3#project_name>`__
            on which to execute the request. The format is:

            ::

                projects/[PROJECT_ID_OR_NUMBER]
        filter (str):
            An optional
            `filter <https://cloud.google.com/monitoring/api/v3/filters>`__
            describing the descriptors to be returned. The filter can
            reference the descriptor's type and labels. For example, the
            following filter returns only Google Compute Engine
            descriptors that have an ``id`` label:

            ::

                resource.type = starts_with("gce_") AND resource.label:id
        page_size (int):
            A positive number that is the maximum number
            of results to return.
        page_token (str):
            If this field is not empty then it must contain the
            ``nextPageToken`` value returned by a previous call to this
            method. Using this field causes the method to return
            additional results from the previous method call.
    """

    name: str = proto.Field(
        proto.STRING,
        number=5,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=2,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=3,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=4,
    )


class ListMonitoredResourceDescriptorsResponse(proto.Message):
    r"""The ``ListMonitoredResourceDescriptors`` response.

    Attributes:
        resource_descriptors (MutableSequence[google.api.monitored_resource_pb2.MonitoredResourceDescriptor]):
            The monitored resource descriptors that are available to
            this project and that match ``filter``, if present.
        next_page_token (str):
            If there are more results than have been returned, then this
            field is set to a non-empty value. To see the additional
            results, use that value as ``page_token`` in the next call
            to this method.
    """

    @property
    def raw_page(self):
        return self

    resource_descriptors: MutableSequence[
        monitored_resource_pb2.MonitoredResourceDescriptor
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=monitored_resource_pb2.MonitoredResourceDescriptor,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class GetMonitoredResourceDescriptorRequest(proto.Message):
    r"""The ``GetMonitoredResourceDescriptor`` request.

    Attributes:
        name (str):
            Required. The monitored resource descriptor to get. The
            format is:

            ::

                projects/[PROJECT_ID_OR_NUMBER]/monitoredResourceDescriptors/[RESOURCE_TYPE]

            The ``[RESOURCE_TYPE]`` is a predefined type, such as
            ``cloudsql_database``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListMetricDescriptorsRequest(proto.Message):
    r"""The ``ListMetricDescriptors`` request.

    Attributes:
        name (str):
            Required. The
            `project <https://cloud.google.com/monitoring/api/v3#project_name>`__
            on which to execute the request. The format is:

            ::

                projects/[PROJECT_ID_OR_NUMBER]
        filter (str):
            Optional. If this field is empty, all custom and
            system-defined metric descriptors are returned. Otherwise,
            the
            `filter <https://cloud.google.com/monitoring/api/v3/filters>`__
            specifies which metric descriptors are to be returned. For
            example, the following filter matches all `custom
            metrics <https://cloud.google.com/monitoring/custom-metrics>`__:

            ::

                metric.type = starts_with("custom.googleapis.com/")
        page_size (int):
            Optional. A positive number that is the maximum number of
            results to return. The default and maximum value is 10,000.
            If a page_size <= 0 or > 10,000 is submitted, will instead
            return a maximum of 10,000 results.
        page_token (str):
            Optional. If this field is not empty then it must contain
            the ``nextPageToken`` value returned by a previous call to
            this method. Using this field causes the method to return
            additional results from the previous method call.
        active_only (bool):
            Optional. If true, only metrics and monitored
            resource types that have recent data (within
            roughly 25 hours) will be included in the
            response.

            - If a metric descriptor enumerates monitored
              resource types, only the monitored resource
              types for which the metric type has recent
              data will be included in the returned
              metric descriptor, and if none of them have
              recent data, the metric descriptor will not be
              returned.
            - If a metric descriptor does not enumerate the
              compatible monitored resource types, it
              will be returned only if the metric type has
              recent data for some monitored resource
              type. The returned descriptor will not
              enumerate any monitored resource types.
    """

    name: str = proto.Field(
        proto.STRING,
        number=5,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=2,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=3,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=4,
    )
    active_only: bool = proto.Field(
        proto.BOOL,
        number=6,
    )


class ListMetricDescriptorsResponse(proto.Message):
    r"""The ``ListMetricDescriptors`` response.

    Attributes:
        metric_descriptors (MutableSequence[google.api.metric_pb2.MetricDescriptor]):
            The metric descriptors that are available to the project and
            that match the value of ``filter``, if present.
        next_page_token (str):
            If there are more results than have been returned, then this
            field is set to a non-empty value. To see the additional
            results, use that value as ``page_token`` in the next call
            to this method.
    """

    @property
    def raw_page(self):
        return self

    metric_descriptors: MutableSequence[
        metric_pb2.MetricDescriptor
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=metric_pb2.MetricDescriptor,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class GetMetricDescriptorRequest(proto.Message):
    r"""The ``GetMetricDescriptor`` request.

    Attributes:
        name (str):
            Required. The metric descriptor on which to execute the
            request. The format is:

            ::

                projects/[PROJECT_ID_OR_NUMBER]/metricDescriptors/[METRIC_ID]

            An example value of ``[METRIC_ID]`` is
            ``"compute.googleapis.com/instance/disk/read_bytes_count"``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=3,
    )


class CreateMetricDescriptorRequest(proto.Message):
    r"""The ``CreateMetricDescriptor`` request.

    Attributes:
        name (str):
            Required. The
            `project <https://cloud.google.com/monitoring/api/v3#project_name>`__
            on which to execute the request. The format is: 4
            projects/[PROJECT_ID_OR_NUMBER]
        metric_descriptor (google.api.metric_pb2.MetricDescriptor):
            Required. The new `custom
            metric <https://cloud.google.com/monitoring/custom-metrics>`__
            descriptor.
    """

    name: str = proto.Field(
        proto.STRING,
        number=3,
    )
    metric_descriptor: metric_pb2.MetricDescriptor = proto.Field(
        proto.MESSAGE,
        number=2,
        message=metric_pb2.MetricDescriptor,
    )


class DeleteMetricDescriptorRequest(proto.Message):
    r"""The ``DeleteMetricDescriptor`` request.

    Attributes:
        name (str):
            Required. The metric descriptor on which to execute the
            request. The format is:

            ::

                projects/[PROJECT_ID_OR_NUMBER]/metricDescriptors/[METRIC_ID]

            An example of ``[METRIC_ID]`` is:
            ``"custom.googleapis.com/my_test_metric"``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListTimeSeriesRequest(proto.Message):
    r"""The ``ListTimeSeries`` request.

    Attributes:
        name (str):
            Required. The
            `project <https://cloud.google.com/monitoring/api/v3#project_name>`__,
            organization or folder on which to execute the request. The
            format is:

            ::

                projects/[PROJECT_ID_OR_NUMBER]
                organizations/[ORGANIZATION_ID]
                folders/[FOLDER_ID]
        filter (str):
            Required. A `monitoring
            filter <https://cloud.google.com/monitoring/api/v3/filters>`__
            that specifies which time series should be returned. The
            filter must specify a single metric type, and can
            additionally specify metric labels and other information.
            For example:

            ::

                metric.type = "compute.googleapis.com/instance/cpu/usage_time" AND
                    metric.labels.instance_name = "my-instance-name".
        interval (google.cloud.monitoring_v3.types.TimeInterval):
            Required. The time interval for which results
            should be returned. Only time series that
            contain data points in the specified interval
            are included in the response.
        aggregation (google.cloud.monitoring_v3.types.Aggregation):
            Specifies the alignment of data points in individual time
            series as well as how to combine the retrieved time series
            across specified labels.

            By default (if no ``aggregation`` is explicitly specified),
            the raw time series data is returned.
        secondary_aggregation (google.cloud.monitoring_v3.types.Aggregation):
            Apply a second aggregation after ``aggregation`` is applied.
            May only be specified if ``aggregation`` is specified.
        order_by (str):
            Unsupported: must be left blank. The points
            in each time series are currently returned in
            reverse time order (most recent to oldest).
        view (google.cloud.monitoring_v3.types.ListTimeSeriesRequest.TimeSeriesView):
            Required. Specifies which information is
            returned about the time series.
        page_size (int):
            A positive number that is the maximum number of results to
            return. If ``page_size`` is empty or more than 100,000
            results, the effective ``page_size`` is 100,000 results. If
            ``view`` is set to ``FULL``, this is the maximum number of
            ``Points`` returned. If ``view`` is set to ``HEADERS``, this
            is the maximum number of ``TimeSeries`` returned.
        page_token (str):
            If this field is not empty then it must contain the
            ``nextPageToken`` value returned by a previous call to this
            method. Using this field causes the method to return
            additional results from the previous method call.
    """

    class TimeSeriesView(proto.Enum):
        r"""Controls which fields are returned by ``ListTimeSeries*``.

        Values:
            FULL (0):
                Returns the identity of the metric(s), the
                time series, and the time series data.
            HEADERS (1):
                Returns the identity of the metric and the
                time series resource, but not the time series
                data.
        """
        FULL = 0
        HEADERS = 1

    name: str = proto.Field(
        proto.STRING,
        number=10,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=2,
    )
    interval: common.TimeInterval = proto.Field(
        proto.MESSAGE,
        number=4,
        message=common.TimeInterval,
    )
    aggregation: common.Aggregation = proto.Field(
        proto.MESSAGE,
        number=5,
        message=common.Aggregation,
    )
    secondary_aggregation: common.Aggregation = proto.Field(
        proto.MESSAGE,
        number=11,
        message=common.Aggregation,
    )
    order_by: str = proto.Field(
        proto.STRING,
        number=6,
    )
    view: TimeSeriesView = proto.Field(
        proto.ENUM,
        number=7,
        enum=TimeSeriesView,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=8,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=9,
    )


class ListTimeSeriesResponse(proto.Message):
    r"""The ``ListTimeSeries`` response.

    Attributes:
        time_series (MutableSequence[google.cloud.monitoring_v3.types.TimeSeries]):
            One or more time series that match the filter
            included in the request.
        next_page_token (str):
            If there are more results than have been returned, then this
            field is set to a non-empty value. To see the additional
            results, use that value as ``page_token`` in the next call
            to this method.
        execution_errors (MutableSequence[google.rpc.status_pb2.Status]):
            Query execution errors that may have caused
            the time series data returned to be incomplete.
        unit (str):
            The unit in which all ``time_series`` point values are
            reported. ``unit`` follows the UCUM format for units as seen
            in https://unitsofmeasure.org/ucum.html. If different
            ``time_series`` have different units (for example, because
            they come from different metric types, or a unit is absent),
            then ``unit`` will be "{not_a_unit}".
    """

    @property
    def raw_page(self):
        return self

    time_series: MutableSequence[gm_metric.TimeSeries] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=gm_metric.TimeSeries,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )
    execution_errors: MutableSequence[status_pb2.Status] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message=status_pb2.Status,
    )
    unit: str = proto.Field(
        proto.STRING,
        number=5,
    )


class CreateTimeSeriesRequest(proto.Message):
    r"""The ``CreateTimeSeries`` request.

    Attributes:
        name (str):
            Required. The
            `project <https://cloud.google.com/monitoring/api/v3#project_name>`__
            on which to execute the request. The format is:

            ::

                projects/[PROJECT_ID_OR_NUMBER]
        time_series (MutableSequence[google.cloud.monitoring_v3.types.TimeSeries]):
            Required. The new data to be added to a list of time series.
            Adds at most one data point to each of several time series.
            The new data point must be more recent than any other point
            in its time series. Each ``TimeSeries`` value must fully
            specify a unique time series by supplying all label values
            for the metric and the monitored resource.

            The maximum number of ``TimeSeries`` objects per ``Create``
            request is 200.
    """

    name: str = proto.Field(
        proto.STRING,
        number=3,
    )
    time_series: MutableSequence[gm_metric.TimeSeries] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message=gm_metric.TimeSeries,
    )


class CreateTimeSeriesError(proto.Message):
    r"""DEPRECATED. Used to hold per-time-series error status.

    Attributes:
        time_series (google.cloud.monitoring_v3.types.TimeSeries):
            DEPRECATED. Time series ID that resulted in the ``status``
            error.
        status (google.rpc.status_pb2.Status):
            DEPRECATED. The status of the requested write operation for
            ``time_series``.
    """

    time_series: gm_metric.TimeSeries = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gm_metric.TimeSeries,
    )
    status: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=2,
        message=status_pb2.Status,
    )


class CreateTimeSeriesSummary(proto.Message):
    r"""Summary of the result of a failed request to write data to a
    time series.

    Attributes:
        total_point_count (int):
            The number of points in the request.
        success_point_count (int):
            The number of points that were successfully
            written.
        errors (MutableSequence[google.cloud.monitoring_v3.types.CreateTimeSeriesSummary.Error]):
            The number of points that failed to be
            written. Order is not guaranteed.
    """

    class Error(proto.Message):
        r"""Detailed information about an error category.

        Attributes:
            status (google.rpc.status_pb2.Status):
                The status of the requested write operation.
            point_count (int):
                The number of points that couldn't be written because of
                ``status``.
        """

        status: status_pb2.Status = proto.Field(
            proto.MESSAGE,
            number=1,
            message=status_pb2.Status,
        )
        point_count: int = proto.Field(
            proto.INT32,
            number=2,
        )

    total_point_count: int = proto.Field(
        proto.INT32,
        number=1,
    )
    success_point_count: int = proto.Field(
        proto.INT32,
        number=2,
    )
    errors: MutableSequence[Error] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message=Error,
    )


class QueryTimeSeriesRequest(proto.Message):
    r"""The ``QueryTimeSeries`` request. For information about the status of
    Monitoring Query Language (MQL), see the `MQL deprecation
    notice <https://cloud.google.com/stackdriver/docs/deprecations/mql>`__.

    Attributes:
        name (str):
            Required. The
            `project <https://cloud.google.com/monitoring/api/v3#project_name>`__
            on which to execute the request. The format is:

            ::

                projects/[PROJECT_ID_OR_NUMBER]
        query (str):
            Required. The query in the `Monitoring Query
            Language <https://cloud.google.com/monitoring/mql/reference>`__
            format. The default time zone is in UTC.
        page_size (int):
            A positive number that is the maximum number of
            time_series_data to return.
        page_token (str):
            If this field is not empty then it must contain the
            ``nextPageToken`` value returned by a previous call to this
            method. Using this field causes the method to return
            additional results from the previous method call.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    query: str = proto.Field(
        proto.STRING,
        number=7,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=9,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=10,
    )


class QueryTimeSeriesResponse(proto.Message):
    r"""The ``QueryTimeSeries`` response. For information about the status
    of Monitoring Query Language (MQL), see the `MQL deprecation
    notice <https://cloud.google.com/stackdriver/docs/deprecations/mql>`__.

    Attributes:
        time_series_descriptor (google.cloud.monitoring_v3.types.TimeSeriesDescriptor):
            The descriptor for the time series data.
        time_series_data (MutableSequence[google.cloud.monitoring_v3.types.TimeSeriesData]):
            The time series data.
        next_page_token (str):
            If there are more results than have been returned, then this
            field is set to a non-empty value. To see the additional
            results, use that value as ``page_token`` in the next call
            to this method.
        partial_errors (MutableSequence[google.rpc.status_pb2.Status]):
            Query execution errors that may have caused
            the time series data returned to be incomplete.
            The available data will be available in the
            response.
    """

    @property
    def raw_page(self):
        return self

    time_series_descriptor: gm_metric.TimeSeriesDescriptor = proto.Field(
        proto.MESSAGE,
        number=8,
        message=gm_metric.TimeSeriesDescriptor,
    )
    time_series_data: MutableSequence[gm_metric.TimeSeriesData] = proto.RepeatedField(
        proto.MESSAGE,
        number=9,
        message=gm_metric.TimeSeriesData,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=10,
    )
    partial_errors: MutableSequence[status_pb2.Status] = proto.RepeatedField(
        proto.MESSAGE,
        number=11,
        message=status_pb2.Status,
    )


class QueryErrorList(proto.Message):
    r"""This is an error detail intended to be used with INVALID_ARGUMENT
    errors.

    Attributes:
        errors (MutableSequence[google.cloud.monitoring_v3.types.QueryError]):
            Errors in parsing the time series query
            language text. The number of errors in the
            response may be limited.
        error_summary (str):
            A summary of all the errors.
    """

    errors: MutableSequence[gm_metric.QueryError] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=gm_metric.QueryError,
    )
    error_summary: str = proto.Field(
        proto.STRING,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.api import label_pb2  # type: ignore
from google.api import launch_stage_pb2  # type: ignore
from google.protobuf import wrappers_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.monitoring_v3.types import common, mutation_record

__protobuf__ = proto.module(
    package="google.monitoring.v3",
    manifest={
        "NotificationChannelDescriptor",
        "NotificationChannel",
    },
)


class NotificationChannelDescriptor(proto.Message):
    r"""A description of a notification channel. The descriptor
    includes the properties of the channel and the set of labels or
    fields that must be specified to configure channels of a given
    type.

    Attributes:
        name (str):
            The full REST resource name for this descriptor. The format
            is:

            ::

                projects/[PROJECT_ID_OR_NUMBER]/notificationChannelDescriptors/[TYPE]

            In the above, ``[TYPE]`` is the value of the ``type`` field.
        type_ (str):
            The type of notification channel, such as "email" and "sms".
            To view the full list of channels, see `Channel
            descriptors <https://cloud.google.com/monitoring/alerts/using-channels-api#ncd>`__.
            Notification channel types are globally unique.
        display_name (str):
            A human-readable name for the notification
            channel type.  This form of the name is suitable
            for a user interface.
        description (str):
            A human-readable description of the
            notification channel type. The description may
            include a description of the properties of the
            channel and pointers to external documentation.
        labels (MutableSequence[google.api.label_pb2.LabelDescriptor]):
            The set of labels that must be defined to
            identify a particular channel of the
            corresponding type. Each label includes a
            description for how that field should be
            populated.
        supported_tiers (MutableSequence[google.cloud.monitoring_v3.types.ServiceTier]):
            The tiers that support this notification channel; the
            project service tier must be one of the supported_tiers.
        launch_stage (google.api.launch_stage_pb2.LaunchStage):
            The product launch stage for channels of this
            type.
    """

    name: str = proto.Field(
        proto.STRING,
        number=6,
    )
    type_: str = proto.Field(
        proto.STRING,
        number=1,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    description: str = proto.Field(
        proto.STRING,
        number=3,
    )
    labels: MutableSequence[label_pb2.LabelDescriptor] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message=label_pb2.LabelDescriptor,
    )
    supported_tiers: MutableSequence[common.ServiceTier] = proto.RepeatedField(
        proto.ENUM,
        number=5,
        enum=common.ServiceTier,
    )
    launch_stage: launch_stage_pb2.LaunchStage = proto.Field(
        proto.ENUM,
        number=7,
        enum=launch_stage_pb2.LaunchStage,
    )


class NotificationChannel(proto.Message):
    r"""A ``NotificationChannel`` is a medium through which an alert is
    delivered when a policy violation is detected. Examples of channels
    include email, SMS, and third-party messaging applications. Fields
    containing sensitive information like authentication tokens or
    contact info are only partially populated on retrieval.

    Attributes:
        type_ (str):
            The type of the notification channel. This field matches the
            value of the
            [NotificationChannelDescriptor.type][google.monitoring.v3.NotificationChannelDescriptor.type]
            field.
        name (str):
            Identifier. The full REST resource name for this channel.
            The format is:

            ::

                projects/[PROJECT_ID_OR_NUMBER]/notificationChannels/[CHANNEL_ID]

            The ``[CHANNEL_ID]`` is automatically assigned by the server
            on creation.
        display_name (str):
            An optional human-readable name for this
            notification channel. It is recommended that you
            specify a non-empty and unique name in order to
            make it easier to identify the channels in your
            project, though this is not enforced. The
            display name is limited to 512 Unicode
            characters.
        description (str):
            An optional human-readable description of
            this notification channel. This description may
            provide additional details, beyond the display
            name, for the channel. This may not exceed 1024
            Unicode characters.
        labels (MutableMapping[str, str]):
            Configuration fields that define the channel and its
            behavior. The permissible and required labels are specified
            in the
            [NotificationChannelDescriptor.labels][google.monitoring.v3.NotificationChannelDescriptor.labels]
            of the ``NotificationChannelDescriptor`` corresponding to
            the ``type`` field.
        user_labels (MutableMapping[str, str]):
            User-supplied key/value data that does not need to conform
            to the corresponding ``NotificationChannelDescriptor``'s
            schema, unlike the ``labels`` field. This field is intended
            to be used for organizing and identifying the
            ``NotificationChannel`` objects.

            The field can contain up to 64 entries. Each key and value
            is limited to 63 Unicode characters or 128 bytes, whichever
            is smaller. Labels and values can contain only lowercase
            letters, numerals, underscores, and dashes. Keys must begin
            with a letter.
        verification_status (google.cloud.monitoring_v3.types.NotificationChannel.VerificationStatus):
            Indicates whether this channel has been verified or not. On
            a
            [``ListNotificationChannels``][google.monitoring.v3.NotificationChannelService.ListNotificationChannels]
            or
            [``GetNotificationChannel``][google.monitoring.v3.NotificationChannelService.GetNotificationChannel]
            operation, this field is expected to be populated.

            If the value is ``UNVERIFIED``, then it indicates that the
            channel is non-functioning (it both requires verification
            and lacks verification); otherwise, it is assumed that the
            channel works.

            If the channel is neither ``VERIFIED`` nor ``UNVERIFIED``,
            it implies that the channel is of a type that does not
            require verification or that this specific channel has been
            exempted from verification because it was created prior to
            verification being required for channels of this type.

            This field cannot be modified using a standard
            [``UpdateNotificationChannel``][google.monitoring.v3.NotificationChannelService.UpdateNotificationChannel]
            operation. To change the value of this field, you must call
            [``VerifyNotificationChannel``][google.monitoring.v3.NotificationChannelService.VerifyNotificationChannel].
        enabled (google.protobuf.wrappers_pb2.BoolValue):
            Whether notifications are forwarded to the
            described channel. This makes it possible to
            disable delivery of notifications to a
            particular channel without removing the channel
            from all alerting policies that reference the
            channel. This is a more convenient approach when
            the change is temporary and you want to receive
            notifications from the same set of alerting
            policies on the channel at some point in the
            future.
        creation_record (google.cloud.monitoring_v3.types.MutationRecord):
            Record of the creation of this channel.
        mutation_records (MutableSequence[google.cloud.monitoring_v3.types.MutationRecord]):
            Records of the modification of this channel.
    """

    class VerificationStatus(proto.Enum):
        r"""Indicates whether the channel has been verified or not. It is
        illegal to specify this field in a
        [``CreateNotificationChannel``][google.monitoring.v3.NotificationChannelService.CreateNotificationChannel]
        or an
        [``UpdateNotificationChannel``][google.monitoring.v3.NotificationChannelService.UpdateNotificationChannel]
        operation.

        Values:
            VERIFICATION_STATUS_UNSPECIFIED (0):
                Sentinel value used to indicate that the
                state is unknown, omitted, or is not applicable
                (as in the case of channels that neither support
                nor require verification in order to function).
            UNVERIFIED (1):
                The channel has yet to be verified and
                requires verification to function. Note that
                this state also applies to the case where the
                verification process has been initiated by
                sending a verification code but where the
                verification code has not been submitted to
                complete the process.
            VERIFIED (2):
                It has been proven that notifications can be
                received on this notification channel and that
                someone on the project has access to messages
                that are delivered to that channel.
        """
        VERIFICATION_STATUS_UNSPECIFIED = 0
        UNVERIFIED = 1
        VERIFIED = 2

    type_: str = proto.Field(
        proto.STRING,
        number=1,
    )
    name: str = proto.Field(
        proto.STRING,
        number=6,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=3,
    )
    description: str = proto.Field(
        proto.STRING,
        number=4,
    )
    labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=5,
    )
    user_labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=8,
    )
    verification_status: VerificationStatus = proto.Field(
        proto.ENUM,
        number=9,
        enum=VerificationStatus,
    )
    enabled: wrappers_pb2.BoolValue = proto.Field(
        proto.MESSAGE,
        number=11,
        message=wrappers_pb2.BoolValue,
    )
    creation_record: mutation_record.MutationRecord = proto.Field(
        proto.MESSAGE,
        number=12,
        message=mutation_record.MutationRecord,
    )
    mutation_records: MutableSequence[
        mutation_record.MutationRecord
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=13,
        message=mutation_record.MutationRecord,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import sys

import google.api_core as api_core

from google.cloud.monitoring_v3 import gapic_version as package_version

__version__ = package_version.__version__

if sys.version_info >= (3, 8):  # pragma: NO COVER
    from importlib import metadata
else:  # pragma: NO COVER
    # TODO(https://github.com/googleapis/python-api-core/issues/835): Remove
    # this code path once we drop support for Python 3.7
    import importlib_metadata as metadata

from .services.alert_policy_service import (
    AlertPolicyServiceAsyncClient,
    AlertPolicyServiceClient,
)
from .services.group_service import GroupServiceAsyncClient, GroupServiceClient
from .services.metric_service import MetricServiceAsyncClient, MetricServiceClient
from .services.notification_channel_service import (
    NotificationChannelServiceAsyncClient,
    NotificationChannelServiceClient,
)
from .services.query_service import QueryServiceAsyncClient, QueryServiceClient
from .services.service_monitoring_service import (
    ServiceMonitoringServiceAsyncClient,
    ServiceMonitoringServiceClient,
)
from .services.snooze_service import SnoozeServiceAsyncClient, SnoozeServiceClient
from .services.uptime_check_service import (
    UptimeCheckServiceAsyncClient,
    UptimeCheckServiceClient,
)
from .types.alert import AlertPolicy
from .types.alert_service import (
    CreateAlertPolicyRequest,
    DeleteAlertPolicyRequest,
    GetAlertPolicyRequest,
    ListAlertPoliciesRequest,
    ListAlertPoliciesResponse,
    UpdateAlertPolicyRequest,
)
from .types.common import (
    Aggregation,
    ComparisonType,
    ServiceTier,
    TimeInterval,
    TypedValue,
)
from .types.dropped_labels import DroppedLabels
from .types.group import Group
from .types.group_service import (
    CreateGroupRequest,
    DeleteGroupRequest,
    GetGroupRequest,
    ListGroupMembersRequest,
    ListGroupMembersResponse,
    ListGroupsRequest,
    ListGroupsResponse,
    UpdateGroupRequest,
)
from .types.metric import (
    LabelValue,
    Point,
    QueryError,
    TextLocator,
    TimeSeries,
    TimeSeriesData,
    TimeSeriesDescriptor,
)
from .types.metric_service import (
    CreateMetricDescriptorRequest,
    CreateTimeSeriesError,
    CreateTimeSeriesRequest,
    CreateTimeSeriesSummary,
    DeleteMetricDescriptorRequest,
    GetMetricDescriptorRequest,
    GetMonitoredResourceDescriptorRequest,
    ListMetricDescriptorsRequest,
    ListMetricDescriptorsResponse,
    ListMonitoredResourceDescriptorsRequest,
    ListMonitoredResourceDescriptorsResponse,
    ListTimeSeriesRequest,
    ListTimeSeriesResponse,
    QueryErrorList,
    QueryTimeSeriesRequest,
    QueryTimeSeriesResponse,
)
from .types.mutation_record import MutationRecord
from .types.notification import NotificationChannel, NotificationChannelDescriptor
from .types.notification_service import (
    CreateNotificationChannelRequest,
    DeleteNotificationChannelRequest,
    GetNotificationChannelDescriptorRequest,
    GetNotificationChannelRequest,
    GetNotificationChannelVerificationCodeRequest,
    GetNotificationChannelVerificationCodeResponse,
    ListNotificationChannelDescriptorsRequest,
    ListNotificationChannelDescriptorsResponse,
    ListNotificationChannelsRequest,
    ListNotificationChannelsResponse,
    SendNotificationChannelVerificationCodeRequest,
    UpdateNotificationChannelRequest,
    VerifyNotificationChannelRequest,
)
from .types.service import (
    BasicSli,
    DistributionCut,
    Range,
    RequestBasedSli,
    Service,
    ServiceLevelIndicator,
    ServiceLevelObjective,
    TimeSeriesRatio,
    WindowsBasedSli,
)
from .types.service_service import (
    CreateServiceLevelObjectiveRequest,
    CreateServiceRequest,
    DeleteServiceLevelObjectiveRequest,
    DeleteServiceRequest,
    GetServiceLevelObjectiveRequest,
    GetServiceRequest,
    ListServiceLevelObjectivesRequest,
    ListServiceLevelObjectivesResponse,
    ListServicesRequest,
    ListServicesResponse,
    UpdateServiceLevelObjectiveRequest,
    UpdateServiceRequest,
)
from .types.snooze import Snooze
from .types.snooze_service import (
    CreateSnoozeRequest,
    GetSnoozeRequest,
    ListSnoozesRequest,
    ListSnoozesResponse,
    UpdateSnoozeRequest,
)
from .types.span_context import SpanContext
from .types.uptime import (
    GroupResourceType,
    InternalChecker,
    SyntheticMonitorTarget,
    UptimeCheckConfig,
    UptimeCheckIp,
    UptimeCheckRegion,
)
from .types.uptime_service import (
    CreateUptimeCheckConfigRequest,
    DeleteUptimeCheckConfigRequest,
    GetUptimeCheckConfigRequest,
    ListUptimeCheckConfigsRequest,
    ListUptimeCheckConfigsResponse,
    ListUptimeCheckIpsRequest,
    ListUptimeCheckIpsResponse,
    UpdateUptimeCheckConfigRequest,
)

if hasattr(api_core, "check_python_version") and hasattr(
    api_core, "check_dependency_versions"
):  # pragma: NO COVER
    api_core.check_python_version("google.cloud.monitoring_v3")  # type: ignore
    api_core.check_dependency_versions("google.cloud.monitoring_v3")  # type: ignore
else:  # pragma: NO COVER
    # An older version of api_core is installed which does not define the
    # functions above. We do equivalent checks manually.
    try:
        import sys
        import warnings

        _py_version_str = sys.version.split()[0]
        _package_label = "google.cloud.monitoring_v3"
        if sys.version_info < (3, 9):
            warnings.warn(
                "You are using a non-supported Python version "
                + f"({_py_version_str}).  Google will not post any further "
                + f"updates to {_package_label} supporting this Python version. "
                + "Please upgrade to the latest Python version, or at "
                + f"least to Python 3.9, and then update {_package_label}.",
                FutureWarning,
            )
        if sys.version_info[:2] == (3, 9):
            warnings.warn(
                f"You are using a Python version ({_py_version_str}) "
                + f"which Google will stop supporting in {_package_label} in "
                + "January 2026. Please "
                + "upgrade to the latest Python version, or at "
                + "least to Python 3.10, before then, and "
                + f"then update {_package_label}.",
                FutureWarning,
            )

        def parse_version_to_tuple(version_string: str):
            """Safely converts a semantic version string to a comparable tuple of integers.
            Example: "4.25.8" -> (4, 25, 8)
            Ignores non-numeric parts and handles common version formats.
            Args:
                version_string: Version string in the format "x.y.z" or "x.y.z<suffix>"
            Returns:
                Tuple of integers for the parsed version string.
            """
            parts = []
            for part in version_string.split("."):
                try:
                    parts.append(int(part))
                except ValueError:
                    # If it's a non-numeric part (e.g., '1.0.0b1' -> 'b1'), stop here.
                    # This is a simplification compared to 'packaging.parse_version', but sufficient
                    # for comparing strictly numeric semantic versions.
                    break
            return tuple(parts)

        def _get_version(dependency_name):
            try:
                version_string: str = metadata.version(dependency_name)
                parsed_version = parse_version_to_tuple(version_string)
                return (parsed_version, version_string)
            except Exception:
                # Catch exceptions from metadata.version() (e.g., PackageNotFoundError)
                # or errors during parse_version_to_tuple
                return (None, "--")

        _dependency_package = "google.protobuf"
        _next_supported_version = "4.25.8"
        _next_supported_version_tuple = (4, 25, 8)
        _recommendation = " (we recommend 6.x)"
        (_version_used, _version_used_string) = _get_version(_dependency_package)
        if _version_used and _version_used < _next_supported_version_tuple:
            warnings.warn(
                f"Package {_package_label} depends on "
                + f"{_dependency_package}, currently installed at version "
                + f"{_version_used_string}. Future updates to "
                + f"{_package_label} will require {_dependency_package} at "
                + f"version {_next_supported_version} or higher{_recommendation}."
                + " Please ensure "
                + "that either (a) your Python environment doesn't pin the "
                + f"version of {_dependency_package}, so that updates to "
                + f"{_package_label} can require the higher version, or "
                + "(b) you manually update your Python environment to use at "
                + f"least version {_next_supported_version} of "
                + f"{_dependency_package}.",
                FutureWarning,
            )
    except Exception:
        warnings.warn(
            "Could not determine the version of Python "
            + "currently being used. To continue receiving "
            + "updates for {_package_label}, ensure you are "
            + "using a supported version of Python; see "
            + "https://devguide.python.org/versions/"
        )

__all__ = (
    "AlertPolicyServiceAsyncClient",
    "GroupServiceAsyncClient",
    "MetricServiceAsyncClient",
    "NotificationChannelServiceAsyncClient",
    "QueryServiceAsyncClient",
    "ServiceMonitoringServiceAsyncClient",
    "SnoozeServiceAsyncClient",
    "UptimeCheckServiceAsyncClient",
    "Aggregation",
    "AlertPolicy",
    "AlertPolicyServiceClient",
    "BasicSli",
    "ComparisonType",
    "CreateAlertPolicyRequest",
    "CreateGroupRequest",
    "CreateMetricDescriptorRequest",
    "CreateNotificationChannelRequest",
    "CreateServiceLevelObjectiveRequest",
    "CreateServiceRequest",
    "CreateSnoozeRequest",
    "CreateTimeSeriesError",
    "CreateTimeSeriesRequest",
    "CreateTimeSeriesSummary",
    "CreateUptimeCheckConfigRequest",
    "DeleteAlertPolicyRequest",
    "DeleteGroupRequest",
    "DeleteMetricDescriptorRequest",
    "DeleteNotificationChannelRequest",
    "DeleteServiceLevelObjectiveRequest",
    "DeleteServiceRequest",
    "DeleteUptimeCheckConfigRequest",
    "DistributionCut",
    "DroppedLabels",
    "GetAlertPolicyRequest",
    "GetGroupRequest",
    "GetMetricDescriptorRequest",
    "GetMonitoredResourceDescriptorRequest",
    "GetNotificationChannelDescriptorRequest",
    "GetNotificationChannelRequest",
    "GetNotificationChannelVerificationCodeRequest",
    "GetNotificationChannelVerificationCodeResponse",
    "GetServiceLevelObjectiveRequest",
    "GetServiceRequest",
    "GetSnoozeRequest",
    "GetUptimeCheckConfigRequest",
    "Group",
    "GroupResourceType",
    "GroupServiceClient",
    "InternalChecker",
    "LabelValue",
    "ListAlertPoliciesRequest",
    "ListAlertPoliciesResponse",
    "ListGroupMembersRequest",
    "ListGroupMembersResponse",
    "ListGroupsRequest",
    "ListGroupsResponse",
    "ListMetricDescriptorsRequest",
    "ListMetricDescriptorsResponse",
    "ListMonitoredResourceDescriptorsRequest",
    "ListMonitoredResourceDescriptorsResponse",
    "ListNotificationChannelDescriptorsRequest",
    "ListNotificationChannelDescriptorsResponse",
    "ListNotificationChannelsRequest",
    "ListNotificationChannelsResponse",
    "ListServiceLevelObjectivesRequest",
    "ListServiceLevelObjectivesResponse",
    "ListServicesRequest",
    "ListServicesResponse",
    "ListSnoozesRequest",
    "ListSnoozesResponse",
    "ListTimeSeriesRequest",
    "ListTimeSeriesResponse",
    "ListUptimeCheckConfigsRequest",
    "ListUptimeCheckConfigsResponse",
    "ListUptimeCheckIpsRequest",
    "ListUptimeCheckIpsResponse",
    "MetricServiceClient",
    "MutationRecord",
    "NotificationChannel",
    "NotificationChannelDescriptor",
    "NotificationChannelServiceClient",
    "Point",
    "QueryError",
    "QueryErrorList",
    "QueryServiceClient",
    "QueryTimeSeriesRequest",
    "QueryTimeSeriesResponse",
    "Range",
    "RequestBasedSli",
    "SendNotificationChannelVerificationCodeRequest",
    "Service",
    "ServiceLevelIndicator",
    "ServiceLevelObjective",
    "ServiceMonitoringServiceClient",
    "ServiceTier",
    "Snooze",
    "SnoozeServiceClient",
    "SpanContext",
    "SyntheticMonitorTarget",
    "TextLocator",
    "TimeInterval",
    "TimeSeries",
    "TimeSeriesData",
    "TimeSeriesDescriptor",
    "TimeSeriesRatio",
    "TypedValue",
    "UpdateAlertPolicyRequest",
    "UpdateGroupRequest",
    "UpdateNotificationChannelRequest",
    "UpdateServiceLevelObjectiveRequest",
    "UpdateServiceRequest",
    "UpdateSnoozeRequest",
    "UpdateUptimeCheckConfigRequest",
    "UptimeCheckConfig",
    "UptimeCheckIp",
    "UptimeCheckRegion",
    "UptimeCheckServiceClient",
    "VerifyNotificationChannelRequest",
    "WindowsBasedSli",
)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import field_mask_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.monitoring_v3.types import snooze as gm_snooze

__protobuf__ = proto.module(
    package="google.monitoring.v3",
    manifest={
        "CreateSnoozeRequest",
        "ListSnoozesRequest",
        "ListSnoozesResponse",
        "GetSnoozeRequest",
        "UpdateSnoozeRequest",
    },
)


class CreateSnoozeRequest(proto.Message):
    r"""The message definition for creating a ``Snooze``. Users must provide
    the body of the ``Snooze`` to be created but must omit the
    ``Snooze`` field, ``name``.

    Attributes:
        parent (str):
            Required. The
            `project <https://cloud.google.com/monitoring/api/v3#project_name>`__
            in which a ``Snooze`` should be created. The format is:

            ::

                projects/[PROJECT_ID_OR_NUMBER]
        snooze (google.cloud.monitoring_v3.types.Snooze):
            Required. The ``Snooze`` to create. Omit the ``name`` field,
            as it will be filled in by the API.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    snooze: gm_snooze.Snooze = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gm_snooze.Snooze,
    )


class ListSnoozesRequest(proto.Message):
    r"""The message definition for listing ``Snooze``\ s associated with the
    given ``parent``, satisfying the optional ``filter``.

    Attributes:
        parent (str):
            Required. The
            `project <https://cloud.google.com/monitoring/api/v3#project_name>`__
            whose ``Snooze``\ s should be listed. The format is:

            ::

                projects/[PROJECT_ID_OR_NUMBER]
        filter (str):
            Optional. Optional filter to restrict results to the given
            criteria. The following fields are supported.

            - ``interval.start_time``
            - ``interval.end_time``

            For example:

            ::

                interval.start_time > "2022-03-11T00:00:00-08:00" AND
                    interval.end_time < "2022-03-12T00:00:00-08:00".
        page_size (int):
            Optional. The maximum number of results to return for a
            single query. The server may further constrain the maximum
            number of results returned in a single page. The value
            should be in the range [1, 1000]. If the value given is
            outside this range, the server will decide the number of
            results to be returned.
        page_token (str):
            Optional. The ``next_page_token`` from a previous call to
            ``ListSnoozesRequest`` to get the next page of results.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=2,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=4,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=5,
    )


class ListSnoozesResponse(proto.Message):
    r"""The results of a successful ``ListSnoozes`` call, containing the
    matching ``Snooze``\ s.

    Attributes:
        snoozes (MutableSequence[google.cloud.monitoring_v3.types.Snooze]):
            ``Snooze``\ s matching this list call.
        next_page_token (str):
            Page token for repeated calls to ``ListSnoozes``, to fetch
            additional pages of results. If this is empty or missing,
            there are no more pages.
    """

    @property
    def raw_page(self):
        return self

    snoozes: MutableSequence[gm_snooze.Snooze] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=gm_snooze.Snooze,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class GetSnoozeRequest(proto.Message):
    r"""The message definition for retrieving a ``Snooze``. Users must
    specify the field, ``name``, which identifies the ``Snooze``.

    Attributes:
        name (str):
            Required. The ID of the ``Snooze`` to retrieve. The format
            is:

            ::

                projects/[PROJECT_ID_OR_NUMBER]/snoozes/[SNOOZE_ID]
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class UpdateSnoozeRequest(proto.Message):
    r"""The message definition for updating a ``Snooze``. The field,
    ``snooze.name`` identifies the ``Snooze`` to be updated. The
    remainder of ``snooze`` gives the content the ``Snooze`` in question
    will be assigned.

    What fields can be updated depends on the start time and end time of
    the ``Snooze``.

    - end time is in the past: These ``Snooze``\ s are considered
      read-only and cannot be updated.
    - start time is in the past and end time is in the future:
      ``display_name`` and ``interval.end_time`` can be updated.
    - start time is in the future: ``display_name``,
      ``interval.start_time`` and ``interval.end_time`` can be updated.

    Attributes:
        snooze (google.cloud.monitoring_v3.types.Snooze):
            Required. The ``Snooze`` to update. Must have the name field
            present.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Required. The fields to update.

            For each field listed in ``update_mask``:

            - If the ``Snooze`` object supplied in the
              ``UpdateSnoozeRequest`` has a value for that field, the
              value of the field in the existing ``Snooze`` will be set
              to the value of the field in the supplied ``Snooze``.
            - If the field does not have a value in the supplied
              ``Snooze``, the field in the existing ``Snooze`` is set to
              its default value.

            Fields not listed retain their existing value.

            The following are the field names that are accepted in
            ``update_mask``:

            - ``display_name``
            - ``interval.start_time``
            - ``interval.end_time``

            That said, the start time and end time of the ``Snooze``
            determines which fields can legally be updated. Before
            attempting an update, users should consult the documentation
            for ``UpdateSnoozeRequest``, which talks about which fields
            can be updated.
    """

    snooze: gm_snooze.Snooze = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gm_snooze.Snooze,
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2016 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import unittest

from google.api import metric_pb2, monitored_resource_pb2
from google.api_core import datetime_helpers
import pandas

from google.cloud import monitoring_v3
from google.cloud.monitoring_v3 import _dataframe

PROJECT = "my-project"

INSTANCE_NAMES = ["instance-1", "instance-2"]
INSTANCE_ZONES = ["us-east1-a", "us-east1-b"]
INSTANCE_IDS = ["1234567890123456789", "9876543210987654321"]

METRIC_TYPE = "compute.googleapis.com/instance/cpu/utilization"
METRIC_LABELS = list({"instance_name": name} for name in INSTANCE_NAMES)

RESOURCE_TYPE = "gce_instance"
RESOURCE_LABELS = list(
    {"project_id": PROJECT, "zone": zone, "instance_id": instance_id}
    for zone, instance_id in zip(INSTANCE_ZONES, INSTANCE_IDS)
)

METRIC_KIND = "GAUGE"
VALUE_TYPE = "DOUBLE"

TIMESTAMPS = [
    "2016-04-06T22:05:00.042Z",
    "2016-04-06T22:05:01.042Z",
    "2016-04-06T22:05:02.042Z",
]

DIMENSIONS = len(TIMESTAMPS), len(INSTANCE_NAMES)
VALUES = list(0.1 * i for i in range(DIMENSIONS[1]))
ARRAY = [VALUES] * DIMENSIONS[0]


def parse_timestamps():
    from google.api_core import datetime_helpers

    return [datetime_helpers.from_rfc3339(t).replace(tzinfo=None) for t in TIMESTAMPS]


def generate_query_results():
    def P(timestamp, value):
        interval = monitoring_v3.TimeInterval()
        interval.start_time = datetime_helpers.from_rfc3339(timestamp).replace(
            tzinfo=None
        )
        interval.end_time = datetime_helpers.from_rfc3339(timestamp).replace(
            tzinfo=None
        )
        return monitoring_v3.Point(interval=interval, value={"double_value": value})

    for metric_labels, resource_labels, value in zip(
        METRIC_LABELS, RESOURCE_LABELS, VALUES
    ):
        yield monitoring_v3.TimeSeries(
            metric=metric_pb2.Metric(type=METRIC_TYPE, labels=metric_labels),
            resource=monitored_resource_pb2.MonitoredResource(
                type=RESOURCE_TYPE, labels=resource_labels
            ),
            metric_kind=METRIC_KIND,
            value_type=VALUE_TYPE,
            points=[P(t, value) for t in TIMESTAMPS],
        )


class Test__build_dataframe(unittest.TestCase):
    def _call_fut(self, *args, **kwargs):
        return _dataframe._build_dataframe(*args, **kwargs)

    def test_both_label_and_labels_illegal(self):
        with self.assertRaises(ValueError):
            self._call_fut([], label="instance_name", labels=["zone"])

    def test_empty_labels_illegal(self):
        with self.assertRaises(ValueError):
            self._call_fut([], labels=[])

    def test_simple_label(self):
        iterable = generate_query_results()
        dataframe = self._call_fut(iterable, label="instance_name")

        self.assertEqual(dataframe.shape, DIMENSIONS)
        self.assertEqual(dataframe.values.tolist(), ARRAY)

        expected_headers = [(instance_name,) for instance_name in INSTANCE_NAMES]
        self.assertEqual(list(dataframe.columns), expected_headers)
        self.assertIsNone(dataframe.columns.name)

        self.assertEqual(list(dataframe.index), parse_timestamps())
        self.assertIsNone(dataframe.index.name)

    def test_multiple_labels(self):
        NAMES = ["resource_type", "instance_id"]

        iterable = generate_query_results()
        dataframe = self._call_fut(iterable, labels=NAMES)

        self.assertEqual(dataframe.shape, DIMENSIONS)
        self.assertEqual(dataframe.values.tolist(), ARRAY)

        expected_headers = [
            (RESOURCE_TYPE, instance_id) for instance_id in INSTANCE_IDS
        ]
        self.assertEqual(list(dataframe.columns), expected_headers)
        self.assertEqual(dataframe.columns.names, NAMES)
        self.assertIsNone(dataframe.columns.name)

        self.assertEqual(list(dataframe.index), parse_timestamps())
        self.assertIsNone(dataframe.index.name)

    def test_multiple_labels_with_just_one(self):
        NAME = "instance_id"
        NAMES = [NAME]

        iterable = generate_query_results()
        dataframe = self._call_fut(iterable, labels=NAMES)

        self.assertEqual(dataframe.shape, DIMENSIONS)
        self.assertEqual(dataframe.values.tolist(), ARRAY)

        expected_headers = [(instance_id,) for instance_id in INSTANCE_IDS]
        self.assertEqual(list(dataframe.columns), expected_headers)
        self.assertEqual(dataframe.columns.names, NAMES)
        self.assertIsNone(dataframe.columns.name)

        self.assertEqual(list(dataframe.index), parse_timestamps())
        self.assertIsNone(dataframe.index.name)

    def test_smart_labels(self):
        NAMES = ["resource_type", "project_id", "zone", "instance_id", "instance_name"]

        iterable = generate_query_results()
        dataframe = self._call_fut(iterable)

        self.assertEqual(dataframe.shape, DIMENSIONS)
        self.assertEqual(dataframe.values.tolist(), ARRAY)

        expected_headers = [
            (RESOURCE_TYPE, PROJECT, zone, instance_id, instance_name)
            for zone, instance_id, instance_name in zip(
                INSTANCE_ZONES, INSTANCE_IDS, INSTANCE_NAMES
            )
        ]
        self.assertEqual(list(dataframe.columns), expected_headers)
        self.assertEqual(dataframe.columns.names, NAMES)
        self.assertIsNone(dataframe.columns.name)

        self.assertEqual(list(dataframe.index), parse_timestamps())
        self.assertIsNone(dataframe.index.name)

    def test_empty_table_simple_label(self):
        dataframe = self._call_fut([], label="instance_name")
        self.assertEqual(dataframe.shape, (0, 0))
        self.assertIsNone(dataframe.columns.name)
        self.assertIsNone(dataframe.index.name)
        self.assertIsInstance(dataframe.index, pandas.DatetimeIndex)

    def test_empty_table_multiple_labels(self):
        NAMES = ["resource_type", "instance_id"]
        dataframe = self._call_fut([], labels=NAMES)
        self.assertEqual(dataframe.shape, (0, 0))
        self.assertEqual(dataframe.columns.names, NAMES)
        self.assertIsNone(dataframe.columns.name)
        self.assertIsNone(dataframe.index.name)
        self.assertIsInstance(dataframe.index, pandas.DatetimeIndex)

    def test_empty_table_multiple_labels_with_just_one(self):
        NAME = "instance_id"
        NAMES = [NAME]
        dataframe = self._call_fut([], labels=NAMES)
        self.assertEqual(dataframe.shape, (0, 0))
        self.assertEqual(dataframe.columns.names, NAMES)
        self.assertIsNone(dataframe.columns.name)
        self.assertIsNone(dataframe.index.name)
        self.assertIsInstance(dataframe.index, pandas.DatetimeIndex)

    def test_empty_table_smart_labels(self):
        NAME = "resource_type"
        NAMES = [NAME]
        dataframe = self._call_fut([])
        self.assertEqual(dataframe.shape, (0, 0))
        self.assertEqual(dataframe.columns.names, NAMES)
        self.assertIsNone(dataframe.columns.name)
        self.assertIsNone(dataframe.index.name)
        self.assertIsInstance(dataframe.index, pandas.DatetimeIndex)


class Test__sorted_resource_labels(unittest.TestCase):
    def _call_fut(self, labels):
        from google.cloud.monitoring_v3._dataframe import _sorted_resource_labels

        return _sorted_resource_labels(labels)

    def test_empty(self):
        self.assertEqual(self._call_fut([]), [])

    def test_sorted(self):
        from google.cloud.monitoring_v3._dataframe import TOP_RESOURCE_LABELS

        EXPECTED = TOP_RESOURCE_LABELS + ("other-1", "other-2")
        self.assertSequenceEqual(self._call_fut(EXPECTED), EXPECTED)

    def test_reversed(self):
        from google.cloud.monitoring_v3._dataframe import TOP_RESOURCE_LABELS

        EXPECTED = TOP_RESOURCE_LABELS + ("other-1", "other-2")
        INPUT = list(reversed(EXPECTED))
        self.assertSequenceEqual(self._call_fut(INPUT), EXPECTED)



# Copyright 2016 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import datetime
import unittest
from unittest import mock

import google.auth.credentials as ga_credentials
import grpc

from google.cloud import monitoring_v3 as monitoring_v3
from google.cloud.monitoring_v3 import MetricServiceClient
from google.cloud.monitoring_v3.services.metric_service.transports import (
    MetricServiceGrpcTransport,
)

PROJECT = "my-project"

METRIC_TYPE = "compute.googleapis.com/instance/uptime"
METRIC_LABELS = {"instance_name": "instance-1"}
METRIC_LABELS2 = {"instance_name": "instance-2"}

RESOURCE_TYPE = "gce_instance"
RESOURCE_LABELS = {
    "project_id": "my-project",
    "zone": "us-east1-a",
    "instance_id": "1234567890123456789",
}
RESOURCE_LABELS2 = {
    "project_id": "my-project",
    "zone": "us-east1-b",
    "instance_id": "9876543210987654321",
}

METRIC_KIND = "DELTA"
VALUE_TYPE = "DOUBLE"

TS0 = datetime.datetime(2016, 4, 6, 22, 5, 0, 42)
TS1 = datetime.datetime(2016, 4, 6, 22, 5, 1, 42)
TS2 = datetime.datetime(2016, 4, 6, 22, 5, 2, 42)


class MultiCallableStub(object):
    """Stub for the grpc.UnaryUnaryMultiCallable interface."""

    def __init__(self, method, channel_stub):
        self.method = method
        self.channel_stub = channel_stub

    def __call__(self, request, timeout=None, metadata=None, credentials=None):
        self.channel_stub.requests.append((self.method, request))

        response = None
        if self.channel_stub.responses:
            response = self.channel_stub.responses.pop()

        if isinstance(response, Exception):
            raise response

        if response:
            return response

    # This method is required; otherwise we see `AttributeError: 'MultiCallableStub' object has no attribute 'with_call'`
    # See https://github.com/grpc/grpc/blob/b53f4055a93fb98601c75dcefaa8f3665167e6cf/src/python/grpcio/grpc/_interceptor.py#L315
    # for the specific location in gRPC code which expects this method.
    def with_call(
        self,
        request,
        timeout=None,
        metadata=None,
        credentials=None,
        wait_for_ready=None,
        compression=None,
    ):
        self.channel_stub.requests.append((self.method, request))
        mock_call = None  # Actual type is grpc.Call but using None since `with_call` is not needed for tests
        response = self.channel_stub.responses.pop()
        if response:
            return (response, mock_call)


class ChannelStub(grpc.Channel):
    """Stub for the grpc.Channel interface."""

    def __init__(self, responses=[]):
        self.responses = responses
        self.responses = [
            monitoring_v3.ListTimeSeriesResponse(**response) for response in responses
        ]
        self.requests = []

    # The method signature should be compatible with this code
    # https://github.com/grpc/grpc/blob/b53f4055a93fb98601c75dcefaa8f3665167e6cf/src/python/grpcio/grpc/_interceptor.py#L696-L701
    # where arguments `method`, `request_serializer`, `response_deserializer` and `_registered_method` are supported
    def unary_unary(
        self,
        method,
        request_serializer=None,
        response_deserializer=None,
        _registered_method=None,
    ):
        return MultiCallableStub(method, self)

    def subscribe(self, callback, try_to_connect=False):
        """grpc.Channel.subscribe implementation."""
        pass

    def unary_stream(
        self,
        method,
        request_serializer=None,
        response_deserializer=None,
    ):
        """grpc.Channel.unary_stream implementation."""
        return self._stub_for_method(method)

    def stream_unary(
        self,
        method,
        request_serializer=None,
        response_deserializer=None,
    ):
        """grpc.Channel.stream_unary implementation."""
        return self._stub_for_method(method)

    def stream_stream(
        self,
        method,
        request_serializer=None,
        response_deserializer=None,
    ):
        """grpc.Channel.stream_stream implementation."""
        return self._stub_for_method(method)

    def unsubscribe(self, callback):
        """grpc.Channel.unsubscribe implementation."""
        pass

    def close(self):
        """grpc.Channel.close implementation."""
        pass


class TestQuery(unittest.TestCase):
    @staticmethod
    def _get_target_class():
        from google.cloud.monitoring_v3.query import Query

        return Query

    def _make_one(self, *args, **kwargs):
        return self._get_target_class()(*args, **kwargs)

    @staticmethod
    def _make_interval(end_time, start_time=None):
        interval = monitoring_v3.TimeInterval(end_time=end_time, start_time=start_time)
        return interval

    @staticmethod
    def _create_client(channel=None):
        if channel is None:
            channel = ChannelStub()
        transport = MetricServiceGrpcTransport(
            channel=channel, credentials=ga_credentials.AnonymousCredentials()
        )
        return MetricServiceClient(transport=transport)

    def test_constructor_minimal(self):
        client = self._create_client()
        query = self._make_one(client, PROJECT)

        self.assertEqual(query._client, client)
        self.assertEqual(
            query._filter.metric_type, self._get_target_class().DEFAULT_METRIC_TYPE
        )

        self.assertIsNone(query._start_time)
        self.assertIsNone(query._end_time)

        self.assertEqual(query._per_series_aligner, 0)
        self.assertEqual(query._alignment_period_seconds, 0)
        self.assertEqual(query._cross_series_reducer, 0)
        self.assertEqual(query._group_by_fields, ())

    def test_constructor_maximal(self):
        T1 = datetime.datetime(2016, 4, 7, 2, 30, 30)
        DAYS, HOURS, MINUTES = 1, 2, 3
        T0 = T1 - datetime.timedelta(days=DAYS, hours=HOURS, minutes=MINUTES)

        client = self._create_client()
        query = self._make_one(
            client,
            PROJECT,
            METRIC_TYPE,
            end_time=T1,
            days=DAYS,
            hours=HOURS,
            minutes=MINUTES,
        )

        self.assertEqual(query._client, client)
        self.assertEqual(query._filter.metric_type, METRIC_TYPE)

        self.assertEqual(query._start_time, T0)
        self.assertEqual(query._end_time, T1)

        self.assertEqual(query._per_series_aligner, 0)
        self.assertEqual(query._alignment_period_seconds, 0)
        self.assertEqual(query._cross_series_reducer, 0)
        self.assertEqual(query._group_by_fields, ())

    def test_constructor_default_end_time(self):
        MINUTES = 5
        NOW = datetime.datetime(2016, 4, 7, 2, 30, 30)
        T0 = datetime.datetime(2016, 4, 7, 2, 25, 0)
        T1 = datetime.datetime(2016, 4, 7, 2, 30, 0)

        client = self._create_client()
        with mock.patch("google.cloud.monitoring_v3.query._UTCNOW", new=lambda: NOW):
            query = self._make_one(client, PROJECT, METRIC_TYPE, minutes=MINUTES)

        self.assertEqual(query._start_time, T0)
        self.assertEqual(query._end_time, T1)

    def test_constructor_nonzero_duration_illegal(self):
        T1 = datetime.datetime(2016, 4, 7, 2, 30, 30)
        client = self._create_client()
        with self.assertRaises(ValueError):
            self._make_one(client, PROJECT, METRIC_TYPE, end_time=T1)

    def test_execution_without_interval_illegal(self):
        client = self._create_client()
        query = self._make_one(client, PROJECT, METRIC_TYPE)
        with self.assertRaises(ValueError):
            list(query)

    def test_metric_type(self):
        client = self._create_client()
        query = self._make_one(client, PROJECT, METRIC_TYPE)
        self.assertEqual(query.metric_type, METRIC_TYPE)

    def test_filter(self):
        client = self._create_client()
        query = self._make_one(client, PROJECT, METRIC_TYPE)
        expected = 'metric.type = "{type}"'.format(type=METRIC_TYPE)
        self.assertEqual(query.filter, expected)

    def test_filter_by_group(self):
        GROUP = "1234567"
        client = self._create_client()
        query = self._make_one(client, PROJECT, METRIC_TYPE)
        query = query.select_group(GROUP)
        expected = ('metric.type = "{type}"' ' AND group.id = "{group}"').format(
            type=METRIC_TYPE, group=GROUP
        )
        self.assertEqual(query.filter, expected)

    def test_filter_by_projects(self):
        PROJECT1, PROJECT2 = "project-1", "project-2"
        client = self._create_client()
        query = self._make_one(client, PROJECT, METRIC_TYPE)
        query = query.select_projects(PROJECT1, PROJECT2)
        expected = (
            'metric.type = "{type}"'
            ' AND project = "{project1}" OR project = "{project2}"'
        ).format(type=METRIC_TYPE, project1=PROJECT1, project2=PROJECT2)
        self.assertEqual(query.filter, expected)

    def test_filter_by_resources(self):
        ZONE_PREFIX = "europe-"
        client = self._create_client()
        query = self._make_one(client, PROJECT, METRIC_TYPE)
        query = query.select_resources(zone_prefix=ZONE_PREFIX)
        expected = (
            'metric.type = "{type}"'
            ' AND resource.label.zone = starts_with("{prefix}")'
        ).format(type=METRIC_TYPE, prefix=ZONE_PREFIX)
        self.assertEqual(query.filter, expected)

    def test_filter_by_metrics(self):
        INSTANCE = "my-instance"
        client = self._create_client()
        query = self._make_one(client, PROJECT, METRIC_TYPE)
        query = query.select_metrics(instance_name=INSTANCE)
        expected = (
            'metric.type = "{type}"' ' AND metric.label.instance_name = "{instance}"'
        ).format(type=METRIC_TYPE, instance=INSTANCE)
        self.assertEqual(query.filter, expected)

    def test_request_parameters_minimal(self):
        T1 = datetime.datetime(2016, 4, 7, 2, 30, 0)

        client = self._create_client()
        query = self._make_one(client, PROJECT, METRIC_TYPE)
        query = query.select_interval(end_time=T1)
        actual = query._build_query_params()
        expected = {
            "name": "projects/{}".format(PROJECT),
            "filter": 'metric.type = "{type}"'.format(type=METRIC_TYPE),
            "interval": self._make_interval(T1),
            "view": monitoring_v3.ListTimeSeriesRequest.TimeSeriesView.FULL,
        }
        self.assertEqual(actual, expected)

    def test_request_parameters_maximal(self):
        T0 = datetime.datetime(2016, 4, 7, 2, 0, 0)
        T1 = datetime.datetime(2016, 4, 7, 2, 30, 0)

        ALIGNER = "ALIGN_DELTA"
        MINUTES, SECONDS, PERIOD_IN_SECONDS = 1, 30, 90

        REDUCER = "REDUCE_MEAN"
        FIELD1, FIELD2 = "resource.zone", "metric.instance_name"

        PAGE_SIZE = 100

        client = self._create_client()
        query = self._make_one(client, PROJECT, METRIC_TYPE)
        query = query.select_interval(start_time=T0, end_time=T1)
        query = query.align(ALIGNER, minutes=MINUTES, seconds=SECONDS)
        query = query.reduce(REDUCER, FIELD1, FIELD2)
        actual = query._build_query_params(headers_only=True, page_size=PAGE_SIZE)
        expected = {
            "name": "projects/%s" % PROJECT,
            "filter": 'metric.type = "{type}"'.format(type=METRIC_TYPE),
            "interval": self._make_interval(T1, T0),
            "aggregation": monitoring_v3.Aggregation(
                per_series_aligner=ALIGNER,
                alignment_period={"seconds": PERIOD_IN_SECONDS},
                cross_series_reducer=REDUCER,
                group_by_fields=[FIELD1, FIELD2],
            ),
            "view": monitoring_v3.ListTimeSeriesRequest.TimeSeriesView.HEADERS,
            "page_size": PAGE_SIZE,
        }
        self.assertEqual(actual, expected)

    def test_iteration(self):
        T0 = datetime.datetime(2016, 4, 6, 22, 5, 0)
        T1 = datetime.datetime(2016, 4, 6, 22, 10, 0)

        INTERVAL1 = self._make_interval(TS1, TS0)
        INTERVAL2 = self._make_interval(TS2, TS1)

        VALUE1 = 60  # seconds
        VALUE2 = 60.001  # seconds

        # Currently cannot create from a list of dict for repeated fields due to
        # https://github.com/googleapis/proto-plus-python/issues/135
        POINT1 = monitoring_v3.Point(
            {"interval": INTERVAL2, "value": {"double_value": VALUE1}}
        )
        POINT2 = monitoring_v3.Point(
            {"interval": INTERVAL1, "value": {"double_value": VALUE1}}
        )
        POINT3 = monitoring_v3.Point(
            {"interval": INTERVAL2, "value": {"double_value": VALUE2}}
        )
        POINT4 = monitoring_v3.Point(
            {"interval": INTERVAL1, "value": {"double_value": VALUE2}}
        )
        SERIES1 = monitoring_v3.TimeSeries(
            {
                "metric": {"type": METRIC_TYPE, "labels": METRIC_LABELS},
                "resource": {"type": RESOURCE_TYPE, "labels": RESOURCE_LABELS},
                "metric_kind": METRIC_KIND,
                "value_type": VALUE_TYPE,
                "points": [POINT1, POINT2],
            }
        )
        SERIES2 = monitoring_v3.TimeSeries(
            {
                "metric": {"type": METRIC_TYPE, "labels": METRIC_LABELS2},
                "resource": {"type": RESOURCE_TYPE, "labels": RESOURCE_LABELS2},
                "metric_kind": METRIC_KIND,
                "value_type": VALUE_TYPE,
                "points": [POINT3, POINT4],
            }
        )

        RESPONSE = {"time_series": [SERIES1, SERIES2], "next_page_token": ""}

        channel = ChannelStub(responses=[RESPONSE])
        client = self._create_client(channel)
        query = self._make_one(client, PROJECT, METRIC_TYPE)
        query = query.select_interval(start_time=T0, end_time=T1)
        response = list(query)

        self.assertEqual(len(response), 2)
        series1, series2 = response

        self.assertEqual(series1.metric.labels, METRIC_LABELS)
        self.assertEqual(series2.metric.labels, METRIC_LABELS2)
        self.assertEqual(series1.resource.labels, RESOURCE_LABELS)
        self.assertEqual(series2.resource.labels, RESOURCE_LABELS2)

        self.assertEqual(
            [p.value.double_value for p in series1.points], [VALUE1, VALUE1]
        )
        self.assertEqual(
            [p.value.double_value for p in series2.points], [VALUE2, VALUE2]
        )
        self.assertEqual([p.interval for p in series1.points], [INTERVAL2, INTERVAL1])
        self.assertEqual([p.interval for p in series2.points], [INTERVAL2, INTERVAL1])

        expected_request = monitoring_v3.ListTimeSeriesRequest(
            name="projects/" + PROJECT,
            filter='metric.type = "{type}"'.format(type=METRIC_TYPE),
            interval=self._make_interval(T1, T0),
            view=monitoring_v3.ListTimeSeriesRequest.TimeSeriesView.FULL,
        )
        request = channel.requests[0][1]
        self.assertEqual(request, expected_request)

    def test_iteration_empty(self):
        T0 = datetime.datetime(2016, 4, 6, 22, 5, 0)
        T1 = datetime.datetime(2016, 4, 6, 22, 10, 0)

        client = self._create_client()
        query = self._make_one(client, PROJECT, METRIC_TYPE)

        with mock.patch.object(
            type(client._transport.list_time_series), "__call__"
        ) as call:
            call.return_value = monitoring_v3.ListTimeSeriesResponse()
            query = query.select_interval(start_time=T0, end_time=T1)
            response = list(query)

            self.assertEqual(len(response), 0)

    def test_iteration_headers_only(self):
        T0 = datetime.datetime(2016, 4, 6, 22, 5, 0)
        T1 = datetime.datetime(2016, 4, 6, 22, 10, 0)

        SERIES1 = {
            "metric": {"type": METRIC_TYPE, "labels": METRIC_LABELS},
            "resource": {"type": RESOURCE_TYPE, "labels": RESOURCE_LABELS},
            "metric_kind": METRIC_KIND,
            "value_type": VALUE_TYPE,
        }
        SERIES2 = {
            "metric": {"type": METRIC_TYPE, "labels": METRIC_LABELS2},
            "resource": {"type": RESOURCE_TYPE, "labels": RESOURCE_LABELS2},
            "metric_kind": METRIC_KIND,
            "value_type": VALUE_TYPE,
        }

        RESPONSE = {"time_series": [SERIES1, SERIES2], "next_page_token": ""}

        channel = ChannelStub(responses=[RESPONSE])
        client = self._create_client(channel)
        query = self._make_one(client, PROJECT, METRIC_TYPE)
        query = query.select_interval(start_time=T0, end_time=T1)

        # add a temporal alignment to test that "aggregation" query params is
        # correctly processed
        query = query.align(monitoring_v3.Aggregation.Aligner.ALIGN_MAX, seconds=3600)
        response = list(query.iter(headers_only=True))

        self.assertEqual(len(response), 2)
        series1, series2 = response

        self.assertEqual(series1.metric.labels, METRIC_LABELS)
        self.assertEqual(series2.metric.labels, METRIC_LABELS2)
        self.assertEqual(series1.resource.labels, RESOURCE_LABELS)
        self.assertEqual(series2.resource.labels, RESOURCE_LABELS2)

        self.assertFalse(len(series1.points))
        self.assertFalse(len(series2.points))

        expected_request = monitoring_v3.ListTimeSeriesRequest(
            name="projects/" + PROJECT,
            filter='metric.type = "{type}"'.format(type=METRIC_TYPE),
            interval=self._make_interval(T1, T0),
            view=monitoring_v3.ListTimeSeriesRequest.TimeSeriesView.HEADERS,
            aggregation=monitoring_v3.Aggregation(
                per_series_aligner=monitoring_v3.Aggregation.Aligner.ALIGN_MAX,
                alignment_period={"seconds": 3600},
            ),
        )
        request = channel.requests[0][1]
        self.assertEqual(request, expected_request)


class Test_Filter(unittest.TestCase):
    @staticmethod
    def _get_target_class():
        from google.cloud.monitoring_v3.query import _Filter

        return _Filter

    def _make_one(self, metric_type):
        return self._get_target_class()(metric_type)

    def test_minimal(self):
        obj = self._make_one(METRIC_TYPE)
        expected = 'metric.type = "{type}"'.format(type=METRIC_TYPE)
        self.assertEqual(str(obj), expected)

    def test_maximal(self):
        obj = self._make_one(METRIC_TYPE)
        obj.group_id = "1234567"
        obj.projects = "project-1", "project-2"
        obj.select_resources(resource_type="some-resource", resource_label="foo")
        obj.select_metrics(metric_label_prefix="bar-")

        expected = (
            'metric.type = "{type}"'
            ' AND group.id = "1234567"'
            ' AND project = "project-1" OR project = "project-2"'
            ' AND resource.label.resource_label = "foo"'
            ' AND resource.type = "some-resource"'
            ' AND metric.label.metric_label = starts_with("bar-")'
        ).format(type=METRIC_TYPE)

        self.assertEqual(str(obj), expected)


class Test__build_label_filter(unittest.TestCase):
    def _call_fut(self, *args, **kwargs):
        from google.cloud.monitoring_v3.query import _build_label_filter

        return _build_label_filter(*args, **kwargs)

    def test_no_labels(self):
        self.assertEqual(self._call_fut("resource"), "")

    def test_label_is_none(self):
        self.assertEqual(self._call_fut("resource", foo=None), "")

    def test_metric_labels(self):
        actual = self._call_fut(
            "metric", alpha_prefix="a-", beta_gamma_suffix="-b", delta_epsilon="xyz"
        )
        expected = (
            'metric.label.alpha = starts_with("a-")'
            ' AND metric.label.beta_gamma = ends_with("-b")'
            ' AND metric.label.delta_epsilon = "xyz"'
        )
        self.assertEqual(actual, expected)

    def test_metric_label_response_code_not_equal(self):
        actual = self._call_fut("metric", response_code_notequal=200)
        expected = "metric.label.response_code != 200"
        self.assertEqual(actual, expected)

    def test_metric_label_response_code_greater_less(self):
        actual = self._call_fut(
            "metric", response_code_greater=500, response_code_less=600
        )
        expected = (
            "metric.label.response_code < 600" " AND metric.label.response_code > 500"
        )
        self.assertEqual(actual, expected)

    def test_metric_label_response_code_greater_less_equal(self):
        actual = self._call_fut(
            "metric", response_code_greaterequal=500, response_code_lessequal=600
        )
        expected = (
            "metric.label.response_code <= 600" " AND metric.label.response_code >= 500"
        )
        self.assertEqual(actual, expected)

    def test_resource_labels(self):
        actual = self._call_fut(
            "resource", alpha_prefix="a-", beta_gamma_suffix="-b", delta_epsilon="xyz"
        )
        expected = (
            'resource.label.alpha = starts_with("a-")'
            ' AND resource.label.beta_gamma = ends_with("-b")'
            ' AND resource.label.delta_epsilon = "xyz"'
        )
        self.assertEqual(actual, expected)

    def test_raw_label_filters(self):
        actual = self._call_fut(
            "resource",
            'resource.label.alpha = starts_with("a-")',
            'resource.label.beta_gamma = ends_with("-b")',
            'resource.label.delta_epsilon = "xyz"',
        )
        expected = (
            'resource.label.alpha = starts_with("a-")'
            ' AND resource.label.beta_gamma = ends_with("-b")'
            ' AND resource.label.delta_epsilon = "xyz"'
        )
        self.assertEqual(actual, expected)

    def test_resource_type(self):
        actual = self._call_fut("resource", resource_type="foo")
        expected = 'resource.type = "foo"'
        self.assertEqual(actual, expected)

    def test_resource_type_prefix(self):
        actual = self._call_fut("resource", resource_type_prefix="foo-")
        expected = 'resource.type = starts_with("foo-")'
        self.assertEqual(actual, expected)

    def test_resource_type_suffix(self):
        actual = self._call_fut("resource", resource_type_suffix="-foo")
        expected = 'resource.type = ends_with("-foo")'
        self.assertEqual(actual, expected)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from google.cloud.monitoring_v3.types import common

__protobuf__ = proto.module(
    package="google.monitoring.v3",
    manifest={
        "Snooze",
    },
)


class Snooze(proto.Message):
    r"""A ``Snooze`` will prevent any alerts from being opened, and close
    any that are already open. The ``Snooze`` will work on alerts that
    match the criteria defined in the ``Snooze``. The ``Snooze`` will be
    active from ``interval.start_time`` through ``interval.end_time``.

    Attributes:
        name (str):
            Required. Identifier. The name of the ``Snooze``. The format
            is:

            ::

                projects/[PROJECT_ID_OR_NUMBER]/snoozes/[SNOOZE_ID]

            The ID of the ``Snooze`` will be generated by the system.
        criteria (google.cloud.monitoring_v3.types.Snooze.Criteria):
            Required. This defines the criteria for applying the
            ``Snooze``. See ``Criteria`` for more information.
        interval (google.cloud.monitoring_v3.types.TimeInterval):
            Required. The ``Snooze`` will be active from
            ``interval.start_time`` through ``interval.end_time``.
            ``interval.start_time`` cannot be in the past. There is a 15
            second clock skew to account for the time it takes for a
            request to reach the API from the UI.
        display_name (str):
            Required. A display name for the ``Snooze``. This can be, at
            most, 512 unicode characters.
    """

    class Criteria(proto.Message):
        r"""Criteria specific to the ``AlertPolicy``\ s that this ``Snooze``
        applies to. The ``Snooze`` will suppress alerts that come from one
        of the ``AlertPolicy``\ s whose names are supplied.

        Attributes:
            policies (MutableSequence[str]):
                The specific ``AlertPolicy`` names for the alert that should
                be snoozed. The format is:

                ::

                    projects/[PROJECT_ID_OR_NUMBER]/alertPolicies/[POLICY_ID]

                There is a limit of 16 policies per snooze. This limit is
                checked during snooze creation. Exactly 1 alert policy is
                required if ``filter`` is specified at the same time.
            filter (str):
                Optional. The filter string to match on Alert fields when
                silencing the alerts. It follows the standard
                https://google.aip.dev/160 syntax. A filter string used to
                apply the snooze to specific incidents that have matching
                filter values. Filters can be defined for snoozes that apply
                to one alerting policy. Filters must be a string formatted
                as one or more resource labels with specific label values.
                If multiple resource labels are used, then they must be
                connected with an AND operator. For example, the following
                filter applies the snooze to incidents that have an instance
                ID of ``1234567890`` and a zone of ``us-central1-a``:

                ::

                    resource.labels.instance_id="1234567890" AND
                    resource.labels.zone="us-central1-a".
        """

        policies: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=1,
        )
        filter: str = proto.Field(
            proto.STRING,
            number=2,
        )

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    criteria: Criteria = proto.Field(
        proto.MESSAGE,
        number=3,
        message=Criteria,
    )
    interval: common.TimeInterval = proto.Field(
        proto.MESSAGE,
        number=4,
        message=common.TimeInterval,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=5,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2016 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Time series query for the `Google Stackdriver Monitoring API (V3)`_.

.. _Google Stackdriver Monitoring API (V3):
    https://cloud.google.com/monitoring/api/ref_v3/rest/v3/\
    projects.timeSeries/list
"""

import copy
import datetime

import google.cloud.monitoring_v3 as monitoring_v3
from google.cloud.monitoring_v3 import _dataframe, types

_UTCNOW = datetime.datetime.utcnow  # To be replaced by tests.


class Query(object):
    """Query object for retrieving metric data.

    :type client: :class:`google.cloud.monitoring_v3.gapic.
        metric_service_client.MetricServiceClient`
    :param client: The client to use.

    :type project: str
    :param project: The project ID or number.

    :type metric_type: str
    :param metric_type: The metric type name. The default value is
        :data:`Query.DEFAULT_METRIC_TYPE
        <google.cloud.monitoring.query.Query.DEFAULT_METRIC_TYPE>`,
        but please note that this default value is provided only for
        demonstration purposes and is subject to change. See the
        `supported metrics`_.

    :type end_time: :class:`datetime.datetime`
    :param end_time: (Optional) The end time (inclusive) of the time interval
        for which results should be returned, as a datetime object.
        The default is the start of the current minute.

        The start time (exclusive) is determined by combining the
        values of  ``days``, ``hours``, and ``minutes``, and
        subtracting the resulting duration from the end time.

        It is also allowed to omit the end time and duration here,
        in which case
        :meth:`~google.cloud.monitoring.query.Query.select_interval`
        must be called before the query is executed.

    :type days: int
    :param days: The number of days in the time interval.

    :type hours: int
    :param hours: The number of hours in the time interval.

    :type minutes: int
    :param minutes: The number of minutes in the time interval.

    :raises: :exc:`ValueError` if ``end_time`` is specified but
        ``days``, ``hours``, and ``minutes`` are all zero.
        If you really want to specify a point in time, use
        :meth:`~google.cloud.monitoring.query.Query.select_interval`.

    .. _supported metrics: https://cloud.google.com/monitoring/api/metrics
    """

    DEFAULT_METRIC_TYPE = "compute.googleapis.com/instance/cpu/utilization"

    def __init__(
        self,
        client,
        project,
        metric_type=DEFAULT_METRIC_TYPE,
        end_time=None,
        days=0,
        hours=0,
        minutes=0,
    ):
        start_time = None
        if days or hours or minutes:
            if end_time is None:
                end_time = _UTCNOW().replace(second=0, microsecond=0)
            start_time = end_time - datetime.timedelta(
                days=days, hours=hours, minutes=minutes
            )
        elif end_time is not None:
            raise ValueError("Non-zero duration required for time interval.")

        self._client = client
        self._project_path = f"projects/{project}"
        self._end_time = end_time
        self._start_time = start_time
        self._filter = _Filter(metric_type)

        self._per_series_aligner = 0
        self._alignment_period_seconds = 0
        self._cross_series_reducer = 0
        self._group_by_fields = ()

    def __iter__(self):
        return self.iter()

    @property
    def metric_type(self):
        """The metric type name."""
        return self._filter.metric_type

    @property
    def filter(self):
        """The filter string.

        This is constructed from the metric type, the resource type, and
        selectors for the group ID, monitored projects, resource labels,
        and metric labels.
        """
        return str(self._filter)

    def select_interval(self, end_time, start_time=None):
        """Copy the query and set the query time interval.

        Example::

            import datetime

            now = datetime.datetime.utcnow()
            query = query.select_interval(
                end_time=now,
                start_time=now - datetime.timedelta(minutes=5))

        As a convenience, you can alternatively specify the end time and
        an interval duration when you create the query initially.

        :type end_time: :class:`datetime.datetime`
        :param end_time: The end time (inclusive) of the time interval
            for which results should be returned, as a datetime object.

        :type start_time: :class:`datetime.datetime`
        :param start_time:
            (Optional) The start time (exclusive) of the time interval
            for which results should be returned, as a datetime object.
            If not specified, the interval is a point in time.

        :rtype: :class:`Query`
        :returns: The new query object.
        """
        new_query = copy.deepcopy(self)
        new_query._end_time = end_time
        new_query._start_time = start_time
        return new_query

    def select_group(self, group_id):
        """Copy the query and add filtering by group.

        Example::

            query = query.select_group('1234567')

        :type group_id: str
        :param group_id: The ID of a group to filter by.

        :rtype: :class:`Query`
        :returns: The new query object.
        """
        new_query = copy.deepcopy(self)
        new_query._filter.group_id = group_id
        return new_query

    def select_projects(self, *args):
        """Copy the query and add filtering by monitored projects.

        This is only useful if the target project represents a Stackdriver
        account containing the specified monitored projects.

        Examples::

            query = query.select_projects('project-1')
            query = query.select_projects('project-1', 'project-2')

        :type args: tuple
        :param args: Project IDs limiting the resources to be included
            in the query.

        :rtype: :class:`Query`
        :returns: The new query object.
        """
        new_query = copy.deepcopy(self)
        new_query._filter.projects = args
        return new_query

    def select_resources(self, *args, **kwargs):
        """Copy the query and add filtering by resource labels.

        See more documentation at: https://cloud.google.com/monitoring/api/v3/filters#comparisons.

        Examples::

            query = query.select_resources(zone='us-central1-a')
            query = query.select_resources(zone_prefix='europe-')
            query = query.select_resources(resource_type='gce_instance')

        A keyword argument ``<label>=<value>`` ordinarily generates a filter
        expression of the form::

            resource.label.<label> = "<value>"

        However, by adding ``"_prefix"`` or ``"_suffix"`` to the keyword,
        you can specify a partial match.

        ``<label>_prefix=<value>`` generates::

            resource.label.<label> = starts_with("<value>")

        ``<label>_suffix=<value>`` generates::

            resource.label.<label> = ends_with("<value>")

        As a special case, ``"resource_type"`` is treated as a special
        pseudo-label corresponding to the filter object ``resource.type``.
        For example, ``resource_type=<value>`` generates::

            resource.type = "<value>"

        See the `defined resource types`_.

        .. note::

            The label ``"instance_name"`` is a metric label,
            not a resource label. You would filter on it using
            ``select_metrics(instance_name=...)``.

        :type args: tuple
        :param args: Raw filter expression strings to include in the
            conjunction. If just one is provided and no keyword arguments
            are provided, it can be a disjunction.

        :param kwargs: Label filters to include in the conjunction as
            described above.

        :rtype: :class:`Query`
        :returns: The new query object.

        .. _defined resource types:
            https://cloud.google.com/monitoring/api/v3/monitored-resources
        """
        new_query = copy.deepcopy(self)
        new_query._filter.select_resources(*args, **kwargs)
        return new_query

    def select_metrics(self, *args, **kwargs):
        """Copy the query and add filtering by metric labels.

        Examples::

            query = query.select_metrics(instance_name='myinstance')
            query = query.select_metrics(instance_name_prefix='mycluster-')

        A keyword argument ``<label>=<value>`` ordinarily generates a filter
        expression of the form::

            metric.label.<label> = "<value>"

        However, by adding ``"_notequal"`` to the keyword, you can inequality:

        ``<label>_notequal=<value>`` generates::

            metric.label.<label> != <value>

        By adding ``"_prefix"`` or ``"_suffix"`` to the keyword, you can specify
        a partial match.

        ``<label>_prefix=<value>`` generates::

            metric.label.<label> = starts_with("<value>")

        ``<label>_suffix=<value>`` generates::

            metric.label.<label> = ends_with("<value>")

        If the label's value type is ``INT64``, a similar notation can be
        used to express inequalities:

        ``<label>_less=<value>`` generates::

            metric.label.<label> < <value>

        ``<label>_lessequal=<value>`` generates::

            metric.label.<label> <= <value>

        ``<label>_greater=<value>`` generates::

            metric.label.<label> > <value>

        ``<label>_greaterequal=<value>`` generates::

            metric.label.<label> >= <value>

        :type args: tuple
        :param args: Raw filter expression strings to include in the
            conjunction. If just one is provided and no keyword arguments
            are provided, it can be a disjunction.

        :param kwargs: Label filters to include in the conjunction as
            described above.

        :rtype: :class:`Query`
        :returns: The new query object.
        """
        new_query = copy.deepcopy(self)
        new_query._filter.select_metrics(*args, **kwargs)
        return new_query

    def align(self, per_series_aligner, seconds=0, minutes=0, hours=0):
        """Copy the query and add temporal alignment.

        If ``per_series_aligner`` is not :data:`Aligner.ALIGN_NONE`, each time
        series will contain data points only on the period boundaries.

        Example::

            from google.cloud import monitoring
            query = query.align(
                monitoring.Aggregation.Aligner.ALIGN_MEAN, minutes=5)

        It is also possible to specify the aligner as a literal string::

            query = query.align('ALIGN_MEAN', minutes=5)

        :type per_series_aligner: str or
            :class:`~google.cloud.monitoring_v3.Aggregation.Aligner`
        :param per_series_aligner: The approach to be used to align
            individual time series. For example: :data:`Aligner.ALIGN_MEAN`.
            See
            :class:`~google.cloud.monitoring_v3.Aggregation.Aligner`
            and the descriptions of the `supported aligners`_.

        :type seconds: int
        :param seconds: The number of seconds in the alignment period.

        :type minutes: int
        :param minutes: The number of minutes in the alignment period.

        :type hours: int
        :param hours: The number of hours in the alignment period.

        :rtype: :class:`Query`
        :returns: The new query object.

        .. _supported aligners:
            https://cloud.google.com/monitoring/api/ref_v3/rest/v3/\
            projects.timeSeries/list#Aligner
        """
        new_query = copy.deepcopy(self)
        new_query._per_series_aligner = per_series_aligner
        new_query._alignment_period_seconds = seconds + 60 * (minutes + 60 * hours)
        return new_query

    def reduce(self, cross_series_reducer, *group_by_fields):
        """Copy the query and add cross-series reduction.

        Cross-series reduction combines time series by aggregating their
        data points.

        For example, you could request an aggregated time series for each
        combination of project and zone as follows::

            from google.cloud import monitoring
            query = query.reduce(monitoring.Aggregation.Reducer.REDUCE_MEAN,
                                 'resource.project_id', 'resource.zone')

        :type cross_series_reducer: str or
            :class:`~google.cloud.monitoring_v3.Aggregation.Reducer`
        :param cross_series_reducer:
            The approach to be used to combine time series. For example:
            :data:`Reducer.REDUCE_MEAN`. See
            :class:`~google.cloud.monitoring_v3.Aggregation.Reducer`
            and the descriptions of the `supported reducers`_.

        :type group_by_fields: strs
        :param group_by_fields:
            Fields to be preserved by the reduction. For example, specifying
            just ``"resource.zone"`` will result in one time series per zone.
            The default is to aggregate all of the time series into just one.

        :rtype: :class:`Query`
        :returns: The new query object.

        .. _supported reducers:
            https://cloud.google.com/monitoring/api/ref_v3/rest/v3/\
            projects.timeSeries/list#Reducer
        """
        new_query = copy.deepcopy(self)
        new_query._cross_series_reducer = cross_series_reducer
        new_query._group_by_fields = group_by_fields
        return new_query

    def iter(self, headers_only=False, page_size=None):
        """Yield all time series objects selected by the query.

        The generator returned iterates over
        :class:`~google.cloud.monitoring_v3.types.TimeSeries` objects
        containing points ordered from oldest to newest.

        Note that the :class:`Query` object itself is an iterable, such that
        the following are equivalent::

            for timeseries in query:
                ...

            for timeseries in query.iter():
                ...

        :type headers_only: bool
        :param headers_only:
             Whether to omit the point data from the time series objects.

        :type page_size: int
        :param page_size:
            (Optional) The maximum number of points in each page of results
            from this request. Non-positive values are ignored. Defaults
            to a sensible value set by the API.

        :raises: :exc:`ValueError` if the query time interval has not been
            specified.
        """
        if self._end_time is None:
            raise ValueError("Query time interval not specified.")

        params = self._build_query_params(headers_only, page_size)

        request = monitoring_v3.ListTimeSeriesRequest(**params)
        for ts in self._client.list_time_series(request):
            yield ts

    def _build_query_params(self, headers_only=False, page_size=None):
        """Return key-value pairs for the list_time_series API call.

        :type headers_only: bool
        :param headers_only:
             Whether to omit the point data from the
             :class:`~google.cloud.monitoring_v3.types.TimeSeries` objects.

        :type page_size: int
        :param page_size:
            (Optional) The maximum number of points in each page of results
            from this request. Non-positive values are ignored. Defaults
            to a sensible value set by the API.
        """
        params = {
            "name": self._project_path,
            "filter": self.filter,
            "interval": types.TimeInterval(
                start_time=self._start_time, end_time=self._end_time
            ),
        }

        if (
            self._per_series_aligner
            or self._alignment_period_seconds
            or self._cross_series_reducer
            or self._group_by_fields
        ):
            params["aggregation"] = types.Aggregation(
                per_series_aligner=self._per_series_aligner,
                cross_series_reducer=self._cross_series_reducer,
                group_by_fields=self._group_by_fields,
                alignment_period={"seconds": self._alignment_period_seconds},
            )

        tsv = monitoring_v3.ListTimeSeriesRequest.TimeSeriesView
        params["view"] = tsv.HEADERS if headers_only else tsv.FULL

        if page_size is not None:
            params["page_size"] = page_size

        return params

    def as_dataframe(self, label=None, labels=None):
        """Return all the selected time series as a :mod:`pandas` dataframe.

        .. note::

            Use of this method requires that you have :mod:`pandas` installed.

        Examples::

            # Generate a dataframe with a multi-level column header including
            # the resource type and all available resource and metric labels.
            # This can be useful for seeing what labels are available.
            dataframe = query.as_dataframe()

            # Generate a dataframe using a particular label for the column
            # names.
            dataframe = query.as_dataframe(label='instance_name')

            # Generate a dataframe with a multi-level column header.
            dataframe = query.as_dataframe(labels=['zone', 'instance_name'])

            # Generate a dataframe with a multi-level column header, assuming
            # the metric is issued by more than one type of resource.
            dataframe = query.as_dataframe(
                labels=['resource_type', 'instance_id'])

        :type label: str
        :param label:
            (Optional) The label name to use for the dataframe header.
            This can be the name of a resource label or metric label
            (e.g., ``"instance_name"``), or the string ``"resource_type"``.

        :type labels: list of strings, or None
        :param labels: A list or tuple of label names to use for the dataframe
            header. If more than one label name is provided, the resulting
            dataframe will have a multi-level column header. Providing values
            for both ``label`` and ``labels`` is an error.

        :rtype: :class:`pandas.DataFrame`
        :returns: A dataframe where each column represents one time series.
        """
        return _dataframe._build_dataframe(self, label, labels)

    def __deepcopy__(self, memo):
        """Create a deepcopy of the query object.

        The `client` attribute is copied by reference only.

        :type memo: dict
        :param memo: the memo dict to avoid excess copying in case  the object
            is referenced from its member.

        :rtype: :class:`Query`
        :returns: The new query object.
        """
        new_query = copy.copy(self)
        new_query._filter = copy.deepcopy(self._filter, memo)
        return new_query


class _Filter(object):
    """Helper for assembling a filter string."""

    def __init__(self, metric_type):
        self.metric_type = metric_type
        self.group_id = None
        self.projects = ()
        self.resource_label_filter = None
        self.metric_label_filter = None

    def select_resources(self, *args, **kwargs):
        """Select by resource labels.

        See :meth:`Query.select_resources`.
        """
        self.resource_label_filter = _build_label_filter("resource", *args, **kwargs)

    def select_metrics(self, *args, **kwargs):
        """Select by metric labels.

        See :meth:`Query.select_metrics`.
        """
        self.metric_label_filter = _build_label_filter("metric", *args, **kwargs)

    def __str__(self):
        filters = ['metric.type = "{type}"'.format(type=self.metric_type)]
        if self.group_id is not None:
            filters.append('group.id = "{id}"'.format(id=self.group_id))
        if self.projects:
            filters.append(
                " OR ".join(
                    'project = "{project}"'.format(project=project)
                    for project in self.projects
                )
            )
        if self.resource_label_filter:
            filters.append(self.resource_label_filter)
        if self.metric_label_filter:
            filters.append(self.metric_label_filter)

        # Parentheses are never actually required, because OR binds more
        # tightly than AND in the Monitoring API's filter syntax.
        return " AND ".join(filters)


def _build_label_filter(category, *args, **kwargs):
    """Construct a filter string to filter on metric or resource labels."""
    terms = list(args)
    for key, value in kwargs.items():
        if value is None:
            continue

        suffix = None
        if key.endswith(
            (
                "_prefix",
                "_suffix",
                "_greater",
                "_greaterequal",
                "_less",
                "_lessequal",
                "_notequal",
            )
        ):
            key, suffix = key.rsplit("_", 1)

        if category == "resource" and key == "resource_type":
            key = "resource.type"
        else:
            key = ".".join((category, "label", key))

        if suffix == "prefix":
            term = '{key} = starts_with("{value}")'
        elif suffix == "suffix":
            term = '{key} = ends_with("{value}")'
        elif suffix == "greater":
            term = "{key} > {value}"
        elif suffix == "greaterequal":
            term = "{key} >= {value}"
        elif suffix == "less":
            term = "{key} < {value}"
        elif suffix == "lessequal":
            term = "{key} <= {value}"
        elif suffix == "notequal":
            term = "{key} != {value}"
        else:
            term = '{key} = "{value}"'

        terms.append(term.format(key=key, value=value))

    return " AND ".join(sorted(terms))
